/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org) 
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509;

import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERSequence;

/**
 * The {@code AccessDescription} object.
 <p>See RFC 3280.</p>
 <p>This class is used by the {@code AuthorityInformationAccess} and
 {@code SubjectInformationAccess} extension.</p>
 <p>ASN.1 definition:</p>
 <pre>
 AccessDescription  ::=  SEQUENCE {
       accessMethod          OBJECT IDENTIFIER,
       accessLocation        GeneralName  }

 id-ad OBJECT IDENTIFIER ::= { id-pkix 48 }

 id-ad-caIssuers OBJECT IDENTIFIER ::= { id-ad 2 }

 id-ad-caRepository OBJECT IDENTIFIER ::= { id-ad 5 }

 id-ad-ocsp OBJECT IDENTIFIER ::= { id-ad 1 }

 id-ad-timeStamping OBJECT IDENTIFIER ::= { id-ad 3 }
 </pre>
 *@see be.cardon.asn1.x509.extensions.AuthorityInformationAccess
 *@see be.cardon.asn1.x509.extensions.SubjectInformationAccess
 *@see GeneralName
 */
public class AccessDescription
    extends ASN1Encodable
{
    /** Objet identifier of {@code id-ad-caIssuers} access method.
   <p>The {@code id-ad-caIssuers} OID is used when the additional information lists
   CAs that have issued certificates superior to the CA that issued the
   certificate containing this extension.  The referenced CA issuers
   description is intended to aid certificate users in the selection of
   a certification path that terminates at a point trusted by the
   certificate user.</p>
   <p>When {@code id-ad-caIssuers} appears as {@code accessMethod}, the 
   {@code accessLocation}
   field describes the referenced description server and the access
   protocol to obtain the referenced description.  The {@code accessLocation}
   field is defined as a {@code GeneralName}, which can take several forms.
   <ul>
   <li>Where the information is available via http, ftp, or ldap,
   {@code accessLocation} MUST be a {@code uniformResourceIdentifier}. </li>
   <li>Where the information is available via the Directory Access Protocol (DAP),
   {@code accessLocation} MUST be a {@code directoryName}.  The entry for that
   {@code directoryName} contains CA certificates in the {@code crossCertificatePair}
   attribute. </li>
    <li>When the information is available via electronic mail,
   {@code accessLocation} MUST be an {@code rfc822Name}.  </li>
    <li>The semantics of other
   {@code id-ad-caIssuers accessLocation} name forms are not defined.</li>
   </ul>
   </p>
     *@see be.cardon.asn1.x509.extensions.AuthorityInformationAccess
     *@see GeneralName
     */
    public final static DERObjectIdentifier id_ad_caIssuers = new DERObjectIdentifier("1.3.6.1.5.5.7.48.2");
    
    /** Objet identifier of {@code id-ad-ocsp} (online certificate status protocol) access method.
   <p>The {@code id-ad-ocsp} OID is used when revocation information for the
   certificate containing this extension is available using the Online
   Certificate Status Protocol (OCSP) [RFC 2560].</p>

   <p>When {@code id-ad-ocsp} appears as accessMethod, the accessLocation field is
   the location of the OCSP responder, using the conventions defined in
   [RFC 2560].</p>
     @see be.cardon.asn1.x509.extensions.AuthorityInformationAccess
     @see GeneralName
     */
    
    public final static DERObjectIdentifier id_ad_ocsp = new DERObjectIdentifier("1.3.6.1.5.5.7.48.1");
    
    
    /** Objet identifier of {@code id-ad-timeStamping} access method.
   <p>The {@code id-ad-timeStamping} OID is used when the subject offers
   timestamping services using the Time Stamp Protocol defined in
   [PKIXTSA].  
   <ul>
   <li>Where the timestamping services are available via http or
   ftp, {@code accessLocation} MUST be a {@code uniformResourceIdentifier}.</li>
   <li>Where the timestamping services are available via electronic mail,
   {@code accessLocation} MUST be an {@code rfc822Name}.  </li>
   <li>Where timestamping services are available using TCP/IP, 
   the {@code dNSName} or {@code ipAddress} name forms may
   be used.  </li>
   <li>The semantics of other name forms of {@code accessLocation} (when
   {@code accessMethod} is {@code id-ad-timeStamping}) are not defined by this
   specification.</li>
   </ul>
   </p>
    @see be.cardon.asn1.x509.extensions.SubjectInformationAccess
    @see GeneralName
     */
    public final static DERObjectIdentifier id_ad_timeStamping = new DERObjectIdentifier("1.3.6.1.5.5.7.48.3");
     
    /** Objet identifier of {@code id-ad-caRepository} access method.
   <p>The {@code id-ad-caRepository} OID is used when the subject is a CA, and
   publishes its certificates and CRLs (if issued) in a repository.  The
   {@code accessLocation} field is defined as a {@code GeneralName}, which can take
   several forms.  
   <ul>
   <li>Where the information is available via http, ftp, or
   ldap, {@code accessLocation} MUST be a {@code uniformResourceIdentifier}.</li>
   <li>Where the information is available via the directory access protocol (dap),
   {@code accessLocation} MUST be a {@code directoryName}.</li>
   <li>When the information is available via electronic mail, 
   {@code accessLocation} MUST be an {@code rfc822Name}.</li>
   <li>The semantics of other name forms of of {@code accessLocation} (when
   {@code accessMethod} is {@code id-ad-caRepository}) are not defined by this
   specification.</li>
   </ul>
   </p>
   @see be.cardon.asn1.x509.extensions.SubjectInformationAccess
    *@see GeneralName
     */
    public final static DERObjectIdentifier id_ad_caRepository = new DERObjectIdentifier("1.3.6.1.5.5.7.48.5");
    
    DERObjectIdentifier accessMethod = null;
    GeneralName accessLocation = null;

    /** Get an instance from a {@code ASN1Sequence} or {@code AccessDescription}
     * object.*/
    public static AccessDescription getInstance(
        Object  obj)
    {
        if (obj instanceof AccessDescription)
        {
            return (AccessDescription)obj;
        }
        else if (obj instanceof ASN1Sequence)
        {
            return new AccessDescription((ASN1Sequence)obj);
        }

        throw new IllegalArgumentException("unknown object in factory");
    }
 
    /** Creates a new instance from a {@code ASN1Sequence} object.*/
    public AccessDescription(
        ASN1Sequence   seq)
    {
        if (seq.size() != 2) 
        {
            throw new IllegalArgumentException("wrong number of elements in inner sequence");
        }
        
        accessMethod = DERObjectIdentifier.getInstance(seq.getObjectAt(0));
        accessLocation = GeneralName.getInstance(seq.getObjectAt(1));
    }

    /**
     * Creates a new instance with the given method access object identifier
     * and the given location.
     */
    public AccessDescription(
        DERObjectIdentifier accessMethod,
        GeneralName accessLocation)
    {
        this.accessMethod = accessMethod;
        this.accessLocation = accessLocation;
    }

    /**
     * Returns the access method.
     */
    public DERObjectIdentifier getAccessMethod()
    {
        return accessMethod;
    }
    
    /**
     * Returns the access location.
     */
    public GeneralName getAccessLocation()
    {
        return accessLocation;
    }
    
    /**
     *@inheritDoc
     */
    public DERObject toASN1Object()
    {
        ASN1EncodableVector accessDescription  = new ASN1EncodableVector();
        
        accessDescription.add(accessMethod);
        accessDescription.add(accessLocation);

        return new DERSequence(accessDescription);
    }
}
