/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org)
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509;

import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.ASN1Set;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERSequence;

/** {@code Attribute} ASN.1 Object.

    <p>Documentation from <b>RFC 3280</b>:</p>
    <p>Attribute data types:</p>
    <pre>

    Attribute       ::=     SEQUENCE {
          type              AttributeType,
          values    SET OF AttributeValue }
                -- at least one value is required

    AttributeType           ::=  OBJECT IDENTIFIER

    AttributeValue          ::=  ANY

    AttributeTypeAndValue           ::=     SEQUENCE {
            type    AttributeType,
            value   AttributeValue }

    </pre>
    <p>Suggested naming attributes: Definition of the following information 
     object set may be augmented to meet local requirements.  Note 
     that deleting members of the set may prevent interoperability with 
     conforming implementations./p>
    <p>Presented in pairs: the {@code AttributeType} followed by the
     type definition for the corresponding {@code AttributeValue}</p>
    <p>Arc for standard naming attributes:</p>
    <pre>
    id-at OBJECT IDENTIFIER ::= { joint-iso-ccitt(2) ds(5) 4 }
    </pre>
    <p>Naming attributes of type X520name:</p>
    <pre>
    id-at-name              AttributeType ::= { id-at 41 }
    id-at-surname           AttributeType ::= { id-at 4 }
    id-at-givenName         AttributeType ::= { id-at 42 }
    id-at-initials          AttributeType ::= { id-at 43 }
    id-at-generationQualifier AttributeType ::= { id-at 44 }

    X520name ::= CHOICE {
          teletexString     TeletexString   (SIZE (1..ub-name)),
          printableString   PrintableString (SIZE (1..ub-name)),
          universalString   UniversalString (SIZE (1..ub-name)),
          utf8String        UTF8String      (SIZE (1..ub-name)),
          bmpString         BMPString       (SIZE (1..ub-name)) }
    </pre>
    <p>Naming attributes of type X520CommonName:</p>
    <pre>
    id-at-commonName        AttributeType ::= { id-at 3 }

    X520CommonName ::= CHOICE {
          teletexString     TeletexString   (SIZE (1..ub-common-name)),
          printableString   PrintableString (SIZE (1..ub-common-name)),
          universalString   UniversalString (SIZE (1..ub-common-name)),
          utf8String        UTF8String      (SIZE (1..ub-common-name)),
          bmpString         BMPString       (SIZE (1..ub-common-name)) }
    </pre>
    <p>Naming attributes of type X520LocalityName:</p>
    <pre>
    id-at-localityName      AttributeType ::= { id-at 7 }

    X520LocalityName ::= CHOICE {
          teletexString     TeletexString   (SIZE (1..ub-locality-name)),
          printableString   PrintableString (SIZE (1..ub-locality-name)),
          universalString   UniversalString (SIZE (1..ub-locality-name)),
          utf8String        UTF8String      (SIZE (1..ub-locality-name)),
          bmpString         BMPString       (SIZE (1..ub-locality-name)) }
    </pre>
    <p>Naming attributes of type X520StateOrProvinceName</p>
    <pre>
    id-at-stateOrProvinceName AttributeType ::= { id-at 8 }

    X520StateOrProvinceName ::= CHOICE {
          teletexString     TeletexString   (SIZE (1..ub-state-name)),
          printableString   PrintableString (SIZE (1..ub-state-name)),
          universalString   UniversalString (SIZE (1..ub-state-name)),
          utf8String        UTF8String      (SIZE (1..ub-state-name)),
          bmpString         BMPString       (SIZE(1..ub-state-name)) }
    </pre>
    <p>Naming attributes of type X520OrganizationName</p>
    <pre>
    id-at-organizationName  AttributeType ::= { id-at 10 }

    X520OrganizationName ::= CHOICE {
          teletexString     TeletexString
                              (SIZE (1..ub-organization-name)),
          printableString   PrintableString
                              (SIZE (1..ub-organization-name)),
          universalString   UniversalString
                              (SIZE (1..ub-organization-name)),
          utf8String        UTF8String
                              (SIZE (1..ub-organization-name)),
          bmpString         BMPString
                              (SIZE (1..ub-organization-name))  }
    </pre>
    <p>Naming attributes of type X520OrganizationalUnitName:</p>
    <pre>
    id-at-organizationalUnitName AttributeType ::= { id-at 11 }

    X520OrganizationalUnitName ::= CHOICE {
          teletexString     TeletexString
                              (SIZE (1..ub-organizational-unit-name)),
          printableString   PrintableString
                              (SIZE (1..ub-organizational-unit-name)),
          universalString   UniversalString
                              (SIZE (1..ub-organizational-unit-name)),
          utf8String        UTF8String
                              (SIZE (1..ub-organizational-unit-name)),
          bmpString         BMPString
                              (SIZE (1..ub-organizational-unit-name)) }
    </pre>
    <p>Naming attributes of type X520Title:</p>
    <pre>
    id-at-title             AttributeType ::= { id-at 12 }

    X520Title ::= CHOICE {
          teletexString     TeletexString   (SIZE (1..ub-title)),
          printableString   PrintableString (SIZE (1..ub-title)),
          universalString   UniversalString (SIZE (1..ub-title)),
          utf8String        UTF8String      (SIZE (1..ub-title)),
          bmpString         BMPString       (SIZE (1..ub-title)) }
    </pre>
    <p>Naming attributes of type X520dnQualifier:</p>
    <pre>
    id-at-dnQualifier       AttributeType ::= { id-at 46 }

    X520dnQualifier ::=     PrintableString
    </pre>
    <p>Naming attributes of type X520countryName (digraph from IS 3166):</p>
    <pre>
    id-at-countryName       AttributeType ::= { id-at 6 }

    X520countryName ::=     PrintableString (SIZE (2))
    </pre>
    <p>Naming attributes of type X520SerialNumber:</p>
    <pre>
    id-at-serialNumber      AttributeType ::= { id-at 5 }

    X520SerialNumber ::=    PrintableString (SIZE (1..ub-serial-number))
    </pre>
    <p>Naming attributes of type X520Pseudonym:</p>
    <pre>
    id-at-pseudonym         AttributeType ::= { id-at 65 }

    X520Pseudonym ::= CHOICE {
       teletexString     TeletexString   (SIZE (1..ub-pseudonym)),
       printableString   PrintableString (SIZE (1..ub-pseudonym)),
       universalString   UniversalString (SIZE (1..ub-pseudonym)),
       utf8String        UTF8String      (SIZE (1..ub-pseudonym)),
       bmpString         BMPString       (SIZE (1..ub-pseudonym)) }
    </pre>
    <p>Naming attributes of type DomainComponent (from RFC 2247):</p>
    <pre>
    id-domainComponent      AttributeType ::=
                              { 0 9 2342 19200300 100 1 25 }

    DomainComponent ::=     IA5String
    </pre>
    <p>Legacy attributes:</p>
    <pre>
    pkcs-9 OBJECT IDENTIFIER ::=
           { iso(1) member-body(2) us(840) rsadsi(113549) pkcs(1) 9 }

    id-emailAddress          AttributeType ::= { pkcs-9 1 }

    EmailAddress ::=         IA5String (SIZE (1..ub-emailaddress-length))
    </pre>
 */
public class Attribute
    extends ASN1Encodable
{
    private DERObjectIdentifier attrType;
    private ASN1Set             attrValues;

    /**
     * Returns an {@code Attribute} object from a {@code ASN1Sequence},
     * or {@code Attribute} object.
     *
     * @param o the object we want converted.
     * @exception IllegalArgumentException if the object cannot be converted.
     */
    public static Attribute getInstance(
        Object o)
    {
        if (o == null || o instanceof Attribute)
        {
            return (Attribute)o;
        }
        
        if (o instanceof ASN1Sequence)
        {
            return new Attribute((ASN1Sequence)o);
        }

        throw new IllegalArgumentException("unknown object in factory");
    }
    
    /**Creates a new instance from a {@code ASN1Sequence} object.*/
    public Attribute(
        ASN1Sequence seq)
    {
        if (seq.size() != 2)
        {
            throw new IllegalArgumentException("Bad sequence size: " + seq.size());
        }

        attrType = DERObjectIdentifier.getInstance(seq.getObjectAt(0));
        attrValues = ASN1Set.getInstance(seq.getObjectAt(1));
    }

    /**Creates a new instance from given {@code attrType} and {@code attrValues}.*/
    public Attribute(
        DERObjectIdentifier attrType,
        ASN1Set             attrValues)
    {
        this.attrType = attrType;
        this.attrValues = attrValues;
    }
    /**Returns the {@code attrType}.*/
    public DERObjectIdentifier getAttrType()
    {
        return attrType;
    }
    
    /**Returns the {@code attrValues}.*/
    public ASN1Set getAttrValues()
    {
        return attrValues;
    }

    /** 
     *@inheritDoc
     */
    public DERObject toASN1Object()
    {
        ASN1EncodableVector v = new ASN1EncodableVector();
        v.add(attrType);
        v.add(attrValues);
        return new DERSequence(v);
    }
}
