/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509;

import java.util.Arrays;
import org.bouncycastle.asn1.*;
import java.util.Map;
import java.util.HashMap;
import java.util.Hashtable;
import be.cardon.utils.Hex;
import java.util.List;


/**{@code AttributeTypeAndValue} {@literal ASN.1} object.
 * *<p>Documentation from <b>RFC 3280</b>:</p>
 *<pre>
 * AttributeTypeAndValue           ::=     SEQUENCE {
 * type    AttributeType,
 * value   AttributeValue }
 *
 * AttributeType           ::=  OBJECT IDENTIFIER
 *
 * AttributeValue          ::=  ANY
 *</pre>
 *<p>See also the {@code Attribute} object :</p>
 *<pre>
 * Attribute       ::=     SEQUENCE {
 * type              AttributeType,
 * values    SET OF AttributeValue }
 * -- at least one value is required
 * </pre>
 */
public class AttributeTypeAndValue extends ASN1Encodable {
    
    /**
     * country code - StringType(SIZE(2))
     */
    public static final DERObjectIdentifier C = new DERObjectIdentifier("2.5.4.6");
    
    /**
     * organization - StringType(SIZE(1..64))
     */
    public static final DERObjectIdentifier O = new DERObjectIdentifier("2.5.4.10");
    
    /**
     * organizational unit name - StringType(SIZE(1..64))
     */
    public static final DERObjectIdentifier OU = new DERObjectIdentifier("2.5.4.11");
    
    /**
     * Title
     */
    public static final DERObjectIdentifier T = new DERObjectIdentifier("2.5.4.12");
    
    /**
     * common name - StringType(SIZE(1..64))
     */
    public static final DERObjectIdentifier CN = new DERObjectIdentifier("2.5.4.3");
    
    /**
     * device serial number name - StringType(SIZE(1..64))
     * Serial
     */
    public static final DERObjectIdentifier SERIALNUMBER = new DERObjectIdentifier("2.5.4.5");
    
    /**
     * street - StringType(SIZE(1..64))
     */
    public static final DERObjectIdentifier STREET = new DERObjectIdentifier("2.5.4.9");
    
    /**
     * device serial number name - StringType(SIZE(1..64))
     */
    // public static final DERObjectIdentifier SN = SERIALNUMBER;
    
    
    /**
     * locality name - StringType(SIZE(1..64))
     */
    public static final DERObjectIdentifier L = new DERObjectIdentifier("2.5.4.7");
    
    /**
     * state, or province name - StringType(SIZE(1..64))
     */
    public static final DERObjectIdentifier ST = new DERObjectIdentifier("2.5.4.8");
    
    /**
     * Naming attributes of type X520name
     */
    public static final DERObjectIdentifier SURNAME = new DERObjectIdentifier("2.5.4.4");
    public static final DERObjectIdentifier GIVENNAME = new DERObjectIdentifier("2.5.4.42");
    public static final DERObjectIdentifier INITIALS = new DERObjectIdentifier("2.5.4.43");
    public static final DERObjectIdentifier GENERATION = new DERObjectIdentifier("2.5.4.44");
    public static final DERObjectIdentifier UNIQUE_IDENTIFIER = new DERObjectIdentifier("2.5.4.45");
    
    /**
     * businessCategory - DirectoryString(SIZE(1..128)
     */
    public static final DERObjectIdentifier BUSINESS_CATEGORY = new DERObjectIdentifier(
            "2.5.4.15");
    
    /**
     * postalCode - DirectoryString(SIZE(1..40)
     */
    public static final DERObjectIdentifier POSTAL_CODE = new DERObjectIdentifier(
            "2.5.4.17");
    
    /**
     * dnQualifier - DirectoryString(SIZE(1..64)
     */
    public static final DERObjectIdentifier DN_QUALIFIER = new DERObjectIdentifier(
            "2.5.4.46");
    
    /**
     * RFC 3039 Pseudonym - DirectoryString(SIZE(1..64)
     */
    public static final DERObjectIdentifier PSEUDONYM = new DERObjectIdentifier(
            "2.5.4.65");
    
    
    /**
     * RFC 3039 DateOfBirth - GeneralizedTime - YYYYMMDD000000Z
     */
    public static final DERObjectIdentifier DATE_OF_BIRTH = new DERObjectIdentifier(
            "1.3.6.1.5.5.7.9.1");
    
    /**
     * RFC 3039 PlaceOfBirth - DirectoryString(SIZE(1..128)
     */
    public static final DERObjectIdentifier PLACE_OF_BIRTH = new DERObjectIdentifier(
            "1.3.6.1.5.5.7.9.2");
    
    /**
     * RFC 3039 Gender - PrintableString (SIZE(1)) -- "M", "F", "m" or "f"
     */
    public static final DERObjectIdentifier GENDER = new DERObjectIdentifier(
            "1.3.6.1.5.5.7.9.3");
    
    /**
     * RFC 3039 CountryOfCitizenship - PrintableString (SIZE (2)) -- ISO 3166
     * codes only
     */
    public static final DERObjectIdentifier COUNTRY_OF_CITIZENSHIP = new DERObjectIdentifier(
            "1.3.6.1.5.5.7.9.4");
    
    /**
     * RFC 3039 CountryOfResidence - PrintableString (SIZE (2)) -- ISO 3166
     * codes only
     */
    public static final DERObjectIdentifier COUNTRY_OF_RESIDENCE = new DERObjectIdentifier(
            "1.3.6.1.5.5.7.9.5");
    
    
    /**
     * ISIS-MTT NameAtBirth - DirectoryString(SIZE(1..64)
     */
    public static final DERObjectIdentifier NAME_AT_BIRTH =  new DERObjectIdentifier("1.3.36.8.3.14");
    
    /**
     * RFC 3039 PostalAddress - SEQUENCE SIZE (1..6) OF
     * DirectoryString(SIZE(1..30))
     */
    public static final DERObjectIdentifier POSTAL_ADDRESS = new DERObjectIdentifier(
            "2.5.4.16");
    
    /*
     * others...
     */
    public static final DERObjectIdentifier DC = new DERObjectIdentifier("0.9.2342.19200300.100.1.25");
    
    /**
     * LDAP User id.
     */
    public static final DERObjectIdentifier UID = new DERObjectIdentifier("0.9.2342.19200300.100.1.1");
    
    public static Map<DERObjectIdentifier,String> DefaultSymbols = new Hashtable<DERObjectIdentifier,String>();
    static{
        
        DefaultSymbols.put(C, "C");
        DefaultSymbols.put(O, "O");
        DefaultSymbols.put(T, "T");
        DefaultSymbols.put(OU, "OU");
        DefaultSymbols.put(CN, "CN");
        DefaultSymbols.put(L, "L");
        DefaultSymbols.put(ST, "ST");
        DefaultSymbols.put(SURNAME, "SN");
        // DefaultSymbols.put(EmailAddress, "E");
        DefaultSymbols.put(DC, "DC");
        DefaultSymbols.put(UID, "UID");
        DefaultSymbols.put(STREET, "STREET");
        DefaultSymbols.put(SURNAME, "SURNAME");
        DefaultSymbols.put(GIVENNAME, "GIVENNAME");
        DefaultSymbols.put(INITIALS, "INITIALS");
        DefaultSymbols.put(GENERATION, "GENERATION");
        // DefaultSymbols.put(UnstructuredAddress, "unstructuredAddress");
        // DefaultSymbols.put(UnstructuredName, "unstructuredName");
        DefaultSymbols.put(UNIQUE_IDENTIFIER, "UniqueIdentifier");
        DefaultSymbols.put(DN_QUALIFIER, "DN");
        DefaultSymbols.put(PSEUDONYM, "Pseudonym");
        DefaultSymbols.put(POSTAL_ADDRESS, "PostalAddress");
        DefaultSymbols.put(NAME_AT_BIRTH, "NameAtBirth");
        DefaultSymbols.put(COUNTRY_OF_CITIZENSHIP, "CountryOfCitizenship");
        DefaultSymbols.put(COUNTRY_OF_RESIDENCE, "CountryOfResidence");
        DefaultSymbols.put(GENDER, "Gender");
        DefaultSymbols.put(PLACE_OF_BIRTH, "PlaceOfBirth");
        DefaultSymbols.put(DATE_OF_BIRTH, "DateOfBirth");
        DefaultSymbols.put(POSTAL_CODE, "PostalCode");
        DefaultSymbols.put(BUSINESS_CATEGORY, "BusinessCategory");
    }
    
    
    
    /**The {@code AttributeType} {@literal ASN.1} Object Identifier. */
    private DERObjectIdentifier attributeType;
    
    /**The {@code AttributeValue} {@literal ASN.1} Object.*/
    private DEREncodable attributeValue;
    
    /** Creates an new object from an existing {@code ASN1Sequence}
     * or {@code AttributeTypeAndValue} object.
     *@throws IllegalArgumentException if the object is invalid.
     */
    public static AttributeTypeAndValue getInstance(
            Object  obj) {
        if (obj instanceof AttributeTypeAndValue) {
            return (AttributeTypeAndValue)obj;
        } else if (obj instanceof ASN1Sequence) {
            return new AttributeTypeAndValue((ASN1Sequence)obj);
        }
        
        throw new IllegalArgumentException("unknown object in factory");
    }
    
    /** Decodes an existing {@code AttributeTypeAndValue} {@literal ASN.1} object.
     *@param seq ASN.1 Sequence being the {code AttributeTypeAndValue}.
     */
    public AttributeTypeAndValue(ASN1Sequence seq) {
        if(seq.size() !=2){
            throw new IllegalArgumentException("AttributeTypeAndValue: bad size sequence" +
                    ", should be 2");
        }
        attributeType = DERObjectIdentifier.getInstance(seq.getObjectAt(0));
        attributeValue = seq.getObjectAt(1);
    }
    
    /**Creates an new instance from a given set of {@code AttributeTypeAndValue}.*/
    public AttributeTypeAndValue(
            DERObjectIdentifier attributeType,
            DEREncodable attributeValue){
        this.attributeType = attributeType;
        this.attributeValue = attributeValue;
    }
    
    /**Returns the {@code AttributeType}.*/
    public DERObjectIdentifier getAttributeType() {
        return attributeType;
    }
    
    /**Returns the {@code AttributeValue}.*/
    public DEREncodable getAttributeValue() {
        return attributeValue;
    }
    
    
    /**
     * {@inheritDoc}
     */
    public DERObject toASN1Object() {
        ASN1EncodableVector  v = new ASN1EncodableVector();
        v.add(attributeType.getDERObject());
        v.add(attributeValue.getDERObject());
        return new DERSequence(v);
    }
    
    /**Returns the RFC2253 string representation (see 2.3).
     *<p>Documentation from RFC2253:</p>
     *<pre>
     * If the AttributeType is in a published table of attribute types
     * associated with LDAP [4], then the type name string from that table
     * is used, otherwise it is encoded as the dotted-decimal encoding of
     * the AttributeType's OBJECT IDENTIFIER. The dotted-decimal notation is
     * described in [3].
     *@param typeMap the published table of attribute types
     * associated with LDAP. If {@code null}, the default map is used.
     */
    public String toRFC2253(Map<DERObjectIdentifier,String> typeMap){
        
        if(typeMap==null){
            typeMap = getRFC2253OIDMap();
        }
        
        //type
        String typeString = typeMap.get(attributeType);
        if(typeString==null){
            typeString = attributeType.getId();
        }
        
        //value
        String valueString;
        if(attributeValue.getDERObject() instanceof DERString){
            String unescaped = ((DERString)attributeValue.getDERObject()).getString();
            List<Character> specials = Arrays.asList(
                    ',','+','"','\\','<','>',';','=');
            String escaped = new String();
            for(int i=0;i<unescaped.length();i++){
                Character c = unescaped.charAt(i);
                if(i==0 && (c =='#' || c ==' ')){
                    escaped += new Character('\\').toString() + c.toString();
                    continue;
                }
                if(i==unescaped.length()-1 && c ==' '){
                    escaped += new Character('\\').toString() + c.toString();
                    continue;
                }
                if(specials.contains(c)){
                    escaped += new Character('\\').toString() + c.toString();
                }else{
                    escaped += c.toString();
                }
            }
            valueString = escaped;
        }else{
            // # with hexadecimal representation
            try{
            valueString = "#" + new String(
                    Hex.encode(attributeValue.getDERObject().getEncoded()));
            }catch(java.io.IOException e){
                e.printStackTrace();
                return null;
            }
        }
        return typeString + "=" + valueString;
    }
    
    /**Returns the RFC2253 string representation using default map(see 2.3).
     *<p>Documentation from RFC2253:</p>
     *<pre>
     * If the AttributeType is in a published table of attribute types
     * associated with LDAP [4], then the type name string from that table
     * is used, otherwise it is encoded as the dotted-decimal encoding of
     * the AttributeType's OBJECT IDENTIFIER. The dotted-decimal notation is
     * described in [3].
     */
    public String toRFC2253(){
        return toRFC2253(null);
    }
    
    /**Returns a non standard string representation.*/
    public String toString(){
        return toRFC2253(getDefaultMap());
    }
    
    /**Returns the map defined in RFC2253.
     *<pre>
     * String  X.500 AttributeType
     * ------------------------------
     * CN      commonName
     * L       localityName
     * ST      stateOrProvinceName
     * O       organizationName
     * OU      organizationalUnitName
     * C       countryName
     * STREET  streetAddress
     * DC      domainComponent
     * UID     userid
     * </pre>
     */
    public static Map<DERObjectIdentifier,String> getRFC2253OIDMap(){
        Map<DERObjectIdentifier,String> map = new HashMap();
        map.put(CN,"CN");
        map.put(L,"L");
        map.put(ST,"ST");
        map.put(O,"O");
        map.put(OU,"OU");
        map.put(C,"C");
        map.put(STREET,"STREET");
        map.put(DC,"DC");
        map.put(UID,"UID");
        return map;
    }
    public static Map<DERObjectIdentifier,String> getDefaultMap(){
        Map<DERObjectIdentifier,String> DefaultSymbols = new HashMap();
        DefaultSymbols.put(C, "C");
        DefaultSymbols.put(O, "O");
        DefaultSymbols.put(T, "T");
        DefaultSymbols.put(OU, "OU");
        DefaultSymbols.put(CN, "CN");
        DefaultSymbols.put(L, "L");
        DefaultSymbols.put(ST, "ST");
        DefaultSymbols.put(SURNAME, "SN");
        // DefaultSymbols.put(EmailAddress, "E");
        DefaultSymbols.put(DC, "DC");
        DefaultSymbols.put(UID, "UID");
        DefaultSymbols.put(STREET, "STREET");
        DefaultSymbols.put(SURNAME, "SURNAME");
        DefaultSymbols.put(GIVENNAME, "GIVENNAME");
        DefaultSymbols.put(SERIALNUMBER, "SERIALNUMBER");
        DefaultSymbols.put(INITIALS, "INITIALS");
        DefaultSymbols.put(GENERATION, "GENERATION");
        // DefaultSymbols.put(UnstructuredAddress, "unstructuredAddress");
        // DefaultSymbols.put(UnstructuredName, "unstructuredName");
        DefaultSymbols.put(UNIQUE_IDENTIFIER, "UniqueIdentifier");
        DefaultSymbols.put(DN_QUALIFIER, "DN");
        DefaultSymbols.put(PSEUDONYM, "Pseudonym");
        DefaultSymbols.put(POSTAL_ADDRESS, "PostalAddress");
        DefaultSymbols.put(NAME_AT_BIRTH, "NameAtBirth");
        DefaultSymbols.put(COUNTRY_OF_CITIZENSHIP, "CountryOfCitizenship");
        DefaultSymbols.put(COUNTRY_OF_RESIDENCE, "CountryOfResidence");
        DefaultSymbols.put(GENDER, "Gender");
        DefaultSymbols.put(PLACE_OF_BIRTH, "PlaceOfBirth");
        DefaultSymbols.put(DATE_OF_BIRTH, "DateOfBirth");
        DefaultSymbols.put(POSTAL_CODE, "PostalCode");
        DefaultSymbols.put(BUSINESS_CATEGORY, "BusinessCategory");
         
        return DefaultSymbols;       
        
    }
}
