/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org)
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509;

import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.x509.X509Name;

/**
 * The {@code GeneralName} object.
 * <p>Implements the {@code GeneralName}{@literal ASN.1} definition from RFC 3280.
 * This class is an enhancement of {@code org.bouncycastle.asn1.x509.GeneralName}.
 * Provides the new function {@code getNameAsString()} for some tags.</p>
 * <pre>
 * GeneralName ::= CHOICE {
 *      otherName                       [0]     OtherName,
 *      rfc822Name                      [1]     IA5String,
 *      dNSName                         [2]     IA5String,
 *      x400Address                     [3]     ORAddress,
 *      directoryName                   [4]     Name,
 *      ediPartyName                    [5]     EDIPartyName,
 *      uniformResourceIdentifier       [6]     IA5String,
 *      iPAddress                       [7]     OCTET STRING,
 *      registeredID                    [8]     OBJECT IDENTIFIER}
 *
 * OtherName ::= SEQUENCE {
 *      type-id    OBJECT IDENTIFIER,
 *      value      [0] EXPLICIT ANY DEFINED BY type-id }
 *
 * EDIPartyName ::= SEQUENCE {
 *      nameAssigner            [0]     DirectoryString OPTIONAL,
 *      partyName               [1]     DirectoryString }
 *
 * Name ::= CHOICE { RDNSequence }
 *</pre>
 *<p>Documentation from RFC 3280 about {@code subjectAltName}</p>
 * <pre>
 * When the subjectAltName extension contains an Internet mail address,
 * the address MUST be included as an rfc822Name. The format of an
 * rfc822Name is an "addr-spec" as defined in RFC 822 [RFC 822].
 *
 * When the subjectAltName extension contains a domain name service
 * label, the domain name MUST be stored in the dNSName (an IA5String).
 * The name MUST be in the "preferred name syntax," as specified by RFC
 * 1034 [RFC 1034].
 *
 * When the subjectAltName extension contains a URI, the name MUST be
 * stored in the uniformResourceIdentifier (an IA5String). The name MUST
 * be a non-relative URL, and MUST follow the URL syntax and encoding
 * rules specified in [RFC 1738].  The name must include both a scheme
 * (e.g., "http" or "ftp") and a scheme-specific-part.  The scheme-
 * specific-part must include a fully qualified domain name or IP
 * address as the host.
 *
 * When the subjectAltName extension contains a iPAddress, the address
 * MUST be stored in the octet string in "network byte order," as
 * specified in RFC 791 [RFC 791]. The least significant bit (LSB) of
 * each octet is the LSB of the corresponding byte in the network
 * address. For IP Version 4, as specified in RFC 791, the octet string
 * MUST contain exactly four octets.  For IP Version 6, as specified in
 * RFC 1883, the octet string MUST contain exactly sixteen octets [RFC
 * 1883].
 *</pre>
 *
 * </pre>
 */
public class GeneralName
        extends ASN1Encodable
        implements ASN1Choice {
    /** Tag {@code otherName} = 0.*/
    public static final int otherName                     = 0;
    
    /** Tag {@code rfc822Name} = 1.*/
    public static final int rfc822Name                    = 1;
    
    /** Tag {@code dNSName} = 2.*/
    public static final int dNSName                       = 2;
    
    /** Tag {@code x400Address} = 3.*/
    public static final int x400Address                   = 3;
    
    /** Tag {@code directoryName} = 4.*/
    public static final int directoryName                 = 4;
    
    /** Tag {@code ediParyName} = 5.*/
    public static final int ediPartyName                  = 5;
    
    /** Tag {@code uniformResourceIdentifier} = 6.*/
    public static final int uniformResourceIdentifier     = 6;
    
    /** Tag {@code iPAddress} = 7.*/
    public static final int iPAddress                     = 7;
    
    /** Tag {@code registeredID} = 8.*/
    public static final int registeredID                  = 8;
    
    DEREncodable      obj;
    int               tag;
    
    /**
     * Constructor for directoryName (tag 4).*/
    public GeneralName(
            X509Name  dirName) {
        this.obj = dirName;
        this.tag = 4;
    }
    
     /**
     * Constructor for directoryName (tag 4).*/
    public GeneralName(
            RDNSequence  dirName) {
        this.obj = dirName;
        this.tag = 4;
    }
    
    /**Default Constructor.
     *
     */
    public GeneralName(
            int           tag,
            ASN1Encodable name) {
        this.obj = name;
        this.tag = tag;
    }
    
    /**Creates a new instance with the given IP address.*/
    public static GeneralName createNewIP(String ipAddress){
        return new GeneralName(iPAddress, ipAddress);
    }
    
    /**
     * Create a General name for the given tag from the passed in String.
     * Supported tags are : rfc822Name, dNSName, uniformResourceIdentifier, registeredID, iPAddress (only v4 yet supported),
     *  and dirName (RDNSequence in format RFC 2253 or 1779).
     *
     * @param tag tag number
     * @param name string representation of name
     */
    public GeneralName(
            int       tag,
            String    name) {
        this.tag = tag;
        if (tag == rfc822Name || tag == dNSName || tag == uniformResourceIdentifier) {
            this.obj = new DERIA5String(name);
        } else if (tag == directoryName) {
            this.obj = new RDNSequence(name);
        } else if (tag == registeredID) {
            this.obj = new DERObjectIdentifier(name);
        } else if (tag == iPAddress){
            String[] nums = name.split("\\p{Punct}");
            if(nums.length!=4){
                throw new IllegalArgumentException("Invalid IPv4, must found 4 addresses");
            }
            byte[] ip = new byte[nums.length];
            Integer num;
            for(int i=0;i<nums.length;i++){
                num = new Integer(nums[i]);
                if(num < 0 || num >255){
                    throw new IllegalArgumentException("Invalid IPv4, number not in [0, 255]");
                }
                ip[i] = num.byteValue();
            }
            this.obj = new DEROctetString(ip);
            
        } else {
            throw new IllegalArgumentException("can't process String for tag: " + tag);
        }
    }
    
    /** General constructor. The object may be a
     * org.bouncycastle.asn1.x509.GeneralName, a be.cardon.asn1.x509.GeneralName,
     * or a ASN1TaggedObject.*/
    public static GeneralName getInstance(
            Object obj) {
        if (obj == null || obj instanceof GeneralName) {
            return (GeneralName)obj;
        }
        if (obj instanceof org.bouncycastle.asn1.x509.GeneralName) {
            org.bouncycastle.asn1.x509.GeneralName gn = (org.bouncycastle.asn1.x509.GeneralName)obj;
            return new GeneralName(
                    gn.getTagNo(),
                    gn.getName().getDERObject());
        }
        if (obj instanceof ASN1TaggedObject) {
            ASN1TaggedObject    tagObj = (ASN1TaggedObject)obj;
            int                 tag = tagObj.getTagNo();
            
            switch (tag) {
                case otherName:
                    return new GeneralName(tag, ASN1Sequence.getInstance(tagObj, false));
                case rfc822Name:
                    return new GeneralName(tag, DERIA5String.getInstance(tagObj, false));
                case dNSName:
                    return new GeneralName(tag, DERIA5String.getInstance(tagObj, false));
                case x400Address:
                    throw new IllegalArgumentException("unknown tag: " + tag);
                case directoryName:
                    return new GeneralName(tag, ASN1Sequence.getInstance(tagObj, true));
                case ediPartyName:
                    return new GeneralName(tag, ASN1Sequence.getInstance(tagObj, false));
                case uniformResourceIdentifier:
                    return new GeneralName(tag, DERIA5String.getInstance(tagObj, false));
                case iPAddress:
                    return new GeneralName(tag, ASN1OctetString.getInstance(tagObj, false));
                case registeredID:
                    return new GeneralName(tag, DERObjectIdentifier.getInstance(tagObj, false));
            }
        }
        
        throw new IllegalArgumentException("unknown object in getInstance");
    }
    
    public static GeneralName getInstance(
            ASN1TaggedObject tagObj,
            boolean          explicit) {
        return GeneralName.getInstance(ASN1TaggedObject.getInstance(tagObj, true));
    }
    
    /** Returns the Tag Number.*/
    public int getTagNo() {
        return tag;
    }
    
    /** Returns the Name as DEREncodable.*/
    public DEREncodable getName() {
        return obj;
    }
    
    /** Returns the Name as String. Supported tag are: rfc822Name,
     * dNSName, directoryName (in format RFC 2253), uniformResourceIdentifier, 
     * registeredID, iPAddress.
     */
    public String getNameAsString(){
        switch (tag) {
            case rfc822Name:
                return DERIA5String.getInstance(obj.getDERObject()).getString();
            case dNSName:
                return DERIA5String.getInstance(obj.getDERObject()).getString();
                //case directoryName:
            case directoryName:
                return RDNSequence.getInstance(obj.getDERObject()).getRFC2253Name();
            case uniformResourceIdentifier:
                return DERIA5String.getInstance(obj.getDERObject()).getString();
                //case iPAddress:
            case registeredID:
                return DERObjectIdentifier.getInstance(obj.getDERObject()).getId();
            case iPAddress:
                byte[] ip = DEROctetString.getInstance(obj.getDERObject()).getOctets();
                if(ip.length==4) {
                    String ipString = new String();
                    for(int i=0;i<ip.length;i++){
                        int num = new Byte(ip[i]).intValue();
                        if(num<0){num+=256;}
                        if(i>0){
                            ipString += "." + num;
                        }else{
                            ipString += num;
                        }
                    }
                    return ipString;
                }else{
                    throw new IllegalArgumentException("only IP version 4 supported.");
                }
                
            default:
                throw new IllegalArgumentException("Tag not supported.");
        }
    }
    public DERObject toASN1Object() {
        if (tag == directoryName)       // directoryName is explicitly tagged as it is a CHOICE
        {
            return new DERTaggedObject(true, tag, obj);
        } else {
            return new DERTaggedObject(false, tag, obj);
        }
    }
    
    
}
