/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 */

package be.cardon.asn1.x509;

import java.util.Enumeration;
import java.util.Vector;
import org.bouncycastle.asn1.*;
import java.util.Iterator;
import java.util.Map;

/**{@literal ASN.1} object {@code RDNSequence}.
 *<p>Documentation from <b>RFC 3280</b>:</p>
 *<pre>
 * Name ::= CHOICE {
     RDNSequence }

   RDNSequence ::= SEQUENCE OF RelativeDistinguishedName
 *</pre>
 * @author crackapat
 */
public class RDNSequence extends ASN1Encodable {
    
    /**Sequence of {@code RelativeDistinguishedName}.*/
    private Vector<RelativeDistinguishedName> rdns;
    
     /** Creates an new object from an existing {@code ASN1Sequence} 
     * or {@code RDNSequence} object.
     *@throws IllegalArgumentException if the object is invalid.
     */
    public static RDNSequence getInstance(
        Object  obj)
    {
        if (obj instanceof RDNSequence)
        {
            return (RDNSequence)obj;
        }
        else if (obj instanceof ASN1Sequence)
        {
            return new RDNSequence((ASN1Sequence)obj);
        }

        throw new IllegalArgumentException("unknown object in factory");
    }
    
     /** Decodes an existing {@code RDNSequence} {@literal ASN.1} object.
     *@param seq ASN.1 Sequence being the {code RDNSequence}.
     */
    public RDNSequence(ASN1Sequence seq) {
        Enumeration e = seq.getObjects();
        rdns = new Vector<RelativeDistinguishedName>();
        while (e.hasMoreElements()){
           rdns.add(RelativeDistinguishedName.getInstance(e.nextElement()));
        }
    }

    /**Creates an new instance from a given vector of {@code RelativeDistinguishedName}.*/
    public RDNSequence(Vector<RelativeDistinguishedName> rdns){
        this.rdns = rdns;
    }
    
    /**Creates an new instance from a given string in format RFC 2253 or 1779}.
     <p>THIS FUNCTION USE THE Sun X500Principal, with limited support for type string representation</p>
     *<p>In the future : own implementation.
     *
     */
    public RDNSequence(String name){
        javax.security.auth.x500.X500Principal xname = new javax.security.auth.x500.X500Principal(name);
        ASN1InputStream in = new ASN1InputStream(new java.io.ByteArrayInputStream(xname.getEncoded()));
        DERObject obj=null;
        try{
        obj = in.readObject();
        }catch(java.io.IOException e){
            e.printStackTrace();
        }
        RDNSequence rdn = RDNSequence.getInstance(obj);
        this.rdns = rdn.getRelativeDistinguishedNames();
    }
        
        
    /**Returns the vector of {@code RelativeDistinguishedName}.*/
    public Vector<RelativeDistinguishedName> getRelativeDistinguishedNames() {
        return rdns;
    }
    
    
    /**
     * {@inheritDoc}
     */    
    public DERObject toASN1Object()
    {
        ASN1EncodableVector  v = new ASN1EncodableVector();
        Iterator<RelativeDistinguishedName> it = rdns.iterator();
        while(it.hasNext()){
            v.add(it.next().getDERObject());
        }
        return new DERSequence(v);
    }
    
    /**Returns the string representation as defined in RFC 2253.
     * <p>See <a href="http://www.ietf.org/rfc/rfc2253.txt">RFC 2253</a>.</p>
     *<p>If the {@code RDNSequence} is an empty sequence, the result is the empty or
        zero length string (not null).</p>
     */
    public String getRFC2253Name(Map<DERObjectIdentifier,String> typeMap){
        if(rdns.size()==0){
            return new String();
        }
        String name = rdns.get(rdns.size()-1).toRFC2253(typeMap);
        for(int i=rdns.size()-2;i>=0;i--){
            name += "," + rdns.get(i).toRFC2253(typeMap);
        }
        return name;
    }
    public String getRFC2253Name(){
        return getRFC2253Name(null);
    }
    public String toString(){
        return getRFC2253Name(AttributeTypeAndValue.getDefaultMap());
    }
    
    /**Creates an new instance from the given string.
     *<p>The string must be in the format of 
     * <a href="http://www.ietf.org/rfc/rfc2253.txt">RFC 2253</a> or 
     * <a href="http://www.ietf.org/rfc/rfc1779.txt">RFC 1779</a>.</p> 
     */
    public static RDNSequence newInstance(String name){
        throw new IllegalArgumentException("not implemented");
    }
    
    
}