/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509;

import java.util.Enumeration;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import org.bouncycastle.asn1.*;
import java.util.Map;


/**{@code RelativeDistinguishedName} {@literal ASN.1} object.
 *<p>Documentation from <b>RFC 3280</b>:</p>
 *<pre>
  RelativeDistinguishedName  ::=
                    SET SIZE (1 .. MAX) OF AttributeTypeAndValue
 *</pre>
 */
public class RelativeDistinguishedName extends ASN1Encodable{

    /**Sequence of {@code AttributeTypeAndValue}.*/
    private Set<AttributeTypeAndValue> atavs;
    
     /** Creates an new object from an existing {@code ASN1Set} 
     * or {@code RelativeDistinguishedName} object.
     *@throws IllegalArgumentException if the object is invalid.
     */
    public static RelativeDistinguishedName getInstance(
        Object  obj)
    {
        if (obj instanceof RelativeDistinguishedName)
        {
            return (RelativeDistinguishedName)obj;
        }
        else if (obj instanceof ASN1Set)
        {
            return new RelativeDistinguishedName((ASN1Set)obj);
        }

        throw new IllegalArgumentException("unknown object in factory");
    }
    
    /** Decodes an existing {@code RelativeDistinguishedName} {@literal ASN.1} object.
    *@param set ASN.1 Set being the {code RelativeDistinguishedName}.
    */
    public RelativeDistinguishedName(ASN1Set set) {
        Enumeration e = set.getObjects();
        atavs = new HashSet<AttributeTypeAndValue>();
        while (e.hasMoreElements()){
           atavs.add(AttributeTypeAndValue.getInstance(e.nextElement()));
        }
    }
    
    /**Creates an new instance from a given set of {@code AttributeTypeAndValue}.*/
    public RelativeDistinguishedName(Set<AttributeTypeAndValue> atavs){
        this.atavs = atavs;
    }

    /**Returns the set of {@code AttributeTypeAndValue}*/
    public Set<AttributeTypeAndValue> getAttributeTypeAndValues() {
        return atavs;
    }
    
     /**
     * {@inheritDoc}
     */    
    public DERObject toASN1Object()
    {
        ASN1EncodableVector  v = new ASN1EncodableVector();
        Iterator<AttributeTypeAndValue> it = atavs.iterator();
        while(it.hasNext()){
            v.add(it.next().getDERObject());
        }
        return new DERSet(v);
    }
    
    /**Returns the RFC2253 string representation (see 2.2).*/
    public String toRFC2253(Map<DERObjectIdentifier,String> typeMap){
        if(atavs.size()<1){
            throw new IllegalArgumentException("AttributeTypeAndValue set is empty.");
        }
        Iterator<AttributeTypeAndValue> it = atavs.iterator();
        String name = new String();
        while(it.hasNext()){
            if(name.length()>0){
                name +="+";
            }
            name += it.next().toRFC2253(typeMap);
        }
        return name;
    }
    
    public String toRFC2253(){
        return toRFC2253(null);
    }
    public String toString(){
        return toRFC2253(AttributeTypeAndValue.getDefaultMap());
    }
    
}
