/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org) 
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509.extensions;

import be.cardon.asn1.x509.GeneralName;
import be.cardon.asn1.x509.AccessDescription;

import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERSequence;

import java.util.Iterator;
import java.util.Vector;

/**Extension {@code AuthorityInformationAccess}.
 
   <p>Documentation from <b>RFC 3280</b>:</p>
 
   <p>The authority information access extension indicates how to access CA
   information and services for the issuer of the certificate in which
   the extension appears.  Information and services may include on-line
   validation services and CA policy data.  (The location of CRLs is not
   specified in this extension; that information is provided by the
   {@code cRLDistributionPoints} extension.)  This extension may be included in
   end entity or CA certificates, and it MUST be non-critical.</p>
   <pre>
   id-pe-authorityInfoAccess OBJECT IDENTIFIER ::= { id-pe 1 }

   AuthorityInfoAccessSyntax  ::=
           SEQUENCE SIZE (1..MAX) OF AccessDescription

   AccessDescription  ::=  SEQUENCE {
           accessMethod          OBJECT IDENTIFIER,
           accessLocation        GeneralName  }

   id-ad OBJECT IDENTIFIER ::= { id-pkix 48 }

   id-ad-caIssuers OBJECT IDENTIFIER ::= { id-ad 2 }

   id-ad-ocsp OBJECT IDENTIFIER ::= { id-ad 1 }
   </pre>

   <p>Each entry in the sequence {@code AuthorityInfoAccessSyntax} describes the
   format and location of additional information provided by the CA that
   issued the certificate in which this extension appears.  The type and
   format of the information is specified by the {@code accessMethod} field; the
   {@code accessLocation} field specifies the location of the information.  The
   retrieval mechanism may be implied by the {@code accessMethod} or specified
   by {@code accessLocation}.</p>

   <p>This profile defines two {@code accessMethod} OIDs: {@code id-ad-caIssuers} and
   {@code id-ad-ocsp}.</p>

   <p>The {@code id-ad-caIssuers} OID is used when the additional information lists
   CAs that have issued certificates superior to the CA that issued the
   certificate containing this extension.  The referenced CA issuers
   description is intended to aid certificate users in the selection of
   a certification path that terminates at a point trusted by the
   certificate user.</p>

   <p>When {@code id-ad-caIssuers} appears as {@code accessMethod}, the 
   {@code accessLocation}
   field describes the referenced description server and the access
   protocol to obtain the referenced description.  The {@code accessLocation}
   field is defined as a {@code GeneralName}, which can take several forms.
   <ul>
   <li>Where the information is available via http, ftp, or ldap,
   {@code accessLocation} MUST be a {@code uniformResourceIdentifier}. </li>
   <li>Where the information is available via the Directory Access Protocol (DAP),
   {@code accessLocation} MUST be a {@code directoryName}.  The entry for that
   {@code directoryName} contains CA certificates in the {@code crossCertificatePair}
   attribute. </li>
    <li>When the information is available via electronic mail,
   {@code accessLocation} MUST be an {@code rfc822Name}.  </li>
    <li>The semantics of other
   {@code id-ad-caIssuers accessLocation} name forms are not defined.</li>
   </ul>
   </p>

   <p>The {@code id-ad-ocsp} OID is used when revocation information for the
   certificate containing this extension is available using the Online
   Certificate Status Protocol (OCSP) [RFC 2560].</p>

   <p>When {@code id-ad-ocsp} appears as accessMethod, the accessLocation field is
   the location of the OCSP responder, using the conventions defined in
   [RFC 2560].</p>

   <p>Additional access descriptors may be defined in other PKIX
   specifications.</p>
 * @see be.cardon.asn1.x509.AccessDescription
 * @author crackapat
 */
public class AuthorityInformationAccess 
    extends ASN1Encodable
{
    private Vector<AccessDescription> accessDescriptionVector;

    /**Returns an instance from a {@code ASN1Sequence}, or a
     *{@code AuthorityInformationAccess} object.*/
    public static AuthorityInformationAccess getInstance(
        Object  obj)
    {
        if (obj instanceof AuthorityInformationAccess)
        {
            return (AuthorityInformationAccess)obj;
        }
        else if (obj instanceof ASN1Sequence)
        {
            return new AuthorityInformationAccess((ASN1Sequence)obj);
        }

        throw new IllegalArgumentException("unknown object in factory");
    }
 
    /**Creates a new instance from a {@code ASN1Sequence} object.*/
    public AuthorityInformationAccess(
        ASN1Sequence   seq)
    {
        accessDescriptionVector = new Vector<AccessDescription>();
        
        for (int i = 0; i != seq.size(); i++)
        {
            accessDescriptionVector.add(AccessDescription.getInstance(seq.getObjectAt(i)));
        }
    }

    /**
     * Creates a new instance with the given {@code accessDescription} vector.
     */
    public AuthorityInformationAccess(
        Vector<AccessDescription> accessDescriptionVector)
    {
        this.accessDescriptionVector = accessDescriptionVector;
    }

    /**Returns a vector with the {@code AccessDescription} objects.*/
    public Vector<AccessDescription> getAccessDescriptionVector() {
        return accessDescriptionVector;
    }

    /**@inheritDoc
     */
    public DERObject toASN1Object()
    {
        ASN1EncodableVector vec = new ASN1EncodableVector();
        Iterator<AccessDescription> it = accessDescriptionVector.iterator();
        while(it.hasNext()){
             vec.add(it.next().toASN1Object());
        }
        
        return new DERSequence(vec);
    }
}

