/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org) 
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509.extensions;


import be.cardon.asn1.x509.*;
import be.cardon.asn1.x509.SubjectPublicKeyInfo;
import java.math.BigInteger;
import java.util.Enumeration;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.bouncycastle.asn1.*;




/**
   Extension {@code AuthorityKeyIdentifier}.

   <p>Documentation from <b>RFC 3280</b>:</p>
   <p>The authority key identifier extension provides a means of
   identifying the public key corresponding to the private key used to
   sign a certificate.  This extension is used where an issuer has
   multiple signing keys (either due to multiple concurrent key pairs or
   due to changeover).  The identification MAY be based on either the
   key identifier (the subject key identifier in the issuer's
   certificate) or on the issuer name and serial number.</p>

   <p>The {@code keyIdentifier} field of the {@code authorityKeyIdentifier}
   extension MUST be included in all certificates generated by conforming CAs to
   facilitate certification path construction.  There is one exception;
   where a CA distributes its public key in the form of a "self-signed"
   certificate, the authority key identifier MAY be omitted.  The
   signature on a self-signed certificate is generated with the private
   key associated with the certificate's subject public key.  (This
   proves that the issuer possesses both the public and private keys.)
   In this case, the subject and authority key identifiers would be
   identical, but only the subject key identifier is needed for
   certification path building.</p>

   <p>The value of the {@code keyIdentifier} field SHOULD be derived from the
   public key used to verify the certificate's signature or a method
   that generates unique values.  Two common methods for generating key
   identifiers from the public key, and one common method for generating
   unique values, are described in section 4.2.1.2 in RFC 3280.  Where a key
   identifier has not been previously established, this specification
   RECOMMENDS use of one of these methods for generating keyIdentifiers.
   Where a key identifier has been previously established, the CA SHOULD
   use the previously established identifier.</p>

   <p>This profile RECOMMENDS support for the key identifier method by all
   certificate users.</p>

   <p>This extension MUST NOT be marked critical.</p>

   <p>The ASN.1 definitions are : </p>
   <pre>
   id-ce-authorityKeyIdentifier OBJECT IDENTIFIER ::=  { id-ce 35 }

   AuthorityKeyIdentifier ::= SEQUENCE {
      keyIdentifier             [0] KeyIdentifier           OPTIONAL,
      authorityCertIssuer       [1] GeneralNames            OPTIONAL,
      authorityCertSerialNumber [2] CertificateSerialNumber OPTIONAL  }

   KeyIdentifier ::= OCTET STRING
   </pre>

 */
public class AuthorityKeyIdentifier
    extends ASN1Encodable
{
    ASN1OctetString keyidentifier=null;
    GeneralNames certissuer=null;
    DERInteger certserno=null;

    /** Creates an new object from an existing {@literal ASN.1} tagged object.*/
    public static AuthorityKeyIdentifier getInstance(
        ASN1TaggedObject obj,
        boolean          explicit)
    {
        return getInstance(ASN1Sequence.getInstance(obj, explicit));
    }

    /** Creates an new object from an existing {@code ASN1Sequence} 
     * or {@code AuthorityKeyIdentifier} object.
     *@throws IllegalArgumentException if the object is invalid.
     */
    public static AuthorityKeyIdentifier getInstance(
        Object  obj)
    {
        if (obj instanceof AuthorityKeyIdentifier)
        {
            return (AuthorityKeyIdentifier)obj;
        }
        else if (obj instanceof ASN1Sequence)
        {
            return new AuthorityKeyIdentifier((ASN1Sequence)obj);
        }

        throw new IllegalArgumentException("unknown object in factory");
    }

    /** Decodes an existing {@code AuthorityKeyIdentifier} {@literal ASN.1} object.
     *@param seq ASN.1 Sequence being the {code AuthorityKeyIdentifier}.
     */
    public AuthorityKeyIdentifier(
        ASN1Sequence   seq)
    {
        Enumeration     e = seq.getObjects();

        while (e.hasMoreElements())
        {
            ASN1TaggedObject o = DERTaggedObject.getInstance(e.nextElement());

            switch (o.getTagNo())
            {
            case 0:
                this.keyidentifier = ASN1OctetString.getInstance(o, false);
                break;
            case 1:
                this.certissuer = GeneralNames.getInstance(o, false);
                break;
            case 2:
                this.certserno = DERInteger.getInstance(o, false);
                break;
            default:
                throw new IllegalArgumentException("illegal tag");
            }
        }
    }

    /**
     *
     * Calulates the {code keyIdentifier} field using a SHA1 hash over the 
     * {@code BIT STRING} from {@link SubjectPublicKeyInfo} as defined in RFC 3280.
     *
     * Example of making a {@code AuthorityKeyIdentifier}:
     * <pre>
     *   SubjectPublicKeyInfo apki = new SubjectPublicKeyInfo((ASN1Sequence)new ASN1InputStream(
     *       publicKey.getEncoded()).readObject());
     *   AuthorityKeyIdentifier aki = new AuthorityKeyIdentifier(apki);
     * </pre>
     *@throws NoSuchAlgorithmException if no SHA-1 {@code MessageDigest} service found.
     *@param spki The {@link SubjectPublicKeyInfo} used to calculates the hash.
     **/
    public AuthorityKeyIdentifier (
        SubjectPublicKeyInfo    spki)
        throws NoSuchAlgorithmException
    {
        MessageDigest digest = MessageDigest.getInstance("SHA-1");
        byte[] bytes = spki.getPublicKeyData().getBytes();
        byte[] resBuf = digest.digest(bytes);
        this.keyidentifier = new DEROctetString(resBuf);
    }

    /**
     * Creates an {@code AuthorityKeyIdentifier} with 
     * the {@code subjectPublicKeyInfo},
     * the {@code authorityCertIssuer} and
     * the {@code authorityCertSerialNumber} provided as well.
     *@throws NoSuchAlgorithmException if no SHA-1 {@code MessageDigest} service found.
     */
    public AuthorityKeyIdentifier(
        SubjectPublicKeyInfo    subjectPublicKeyInfo,
        GeneralNames            authorityCertIssuer,
        BigInteger              authorityCertSerialNumber) 
        throws NoSuchAlgorithmException
    {
        MessageDigest digest = MessageDigest.getInstance("SHA-1");
        byte[] bytes = subjectPublicKeyInfo.getPublicKeyData().getBytes();
        byte[] resBuf = digest.digest(bytes);
        
        this.keyidentifier = new DEROctetString(resBuf);
        this.certissuer = authorityCertIssuer;
        this.certserno = new DERInteger(authorityCertSerialNumber);
    }

    /**
     * Creates an {@code AuthorityKeyIdentifier} with 
     * the {@code authorityCertIssuer} and the {@code authorityCertSerialNumber} provided.
     */
    public AuthorityKeyIdentifier(
        GeneralNames            authorityCertIssuer,
        BigInteger              authorityCertSerialNumber)
    {
        this.keyidentifier = null;
        this.certissuer = GeneralNames.getInstance(authorityCertIssuer.toASN1Object());
        this.certserno = new DERInteger(authorityCertSerialNumber);
    }

    /**
     * Creates an {@code AuthorityKeyIdentifier} with a precomputed key identifier,
     * the {@code authorityCertIssuer} and the serial number provided as well.
     */
    public AuthorityKeyIdentifier(
        byte[]                  keyIdentifier,
        GeneralNames            authorityCertIssuer,
        BigInteger              authorityCertSerialNumber)
    {
        this.keyidentifier = new DEROctetString(keyIdentifier);
        this.certissuer = GeneralNames.getInstance(authorityCertIssuer.toASN1Object());
        this.certserno = new DERInteger(authorityCertSerialNumber);
    }
    
    /**Returns the {@code keyIdentifier} element
     * or {@code null} it absent.*/
    public byte[] getKeyIdentifier()
    {
        if (keyidentifier != null)
        {
            return keyidentifier.getOctets();
        }

        return null;
    }
    
   /**Returns the {@code authorityCertIssuer} element 
    * or {@code null} if absent.*/
    public GeneralNames getAuthorityCertIssuer()
    {
        return certissuer;
    }
    
    /**Returns the {@code AuthorityCertSerialNumber} element 
    * or {@code null} if absent.*/ 
    public BigInteger getAuthorityCertSerialNumber()
    {
        if (certserno != null)
        {
            return certserno.getValue();
        }
        
        return null;
    }
    
    /**
     * {@inheritDoc}
     */
    public DERObject toASN1Object()
    {
        ASN1EncodableVector  v = new ASN1EncodableVector();

        if (keyidentifier != null)
        {
            v.add(new DERTaggedObject(false, 0, keyidentifier));
        }

        if (certissuer != null)
        {
            v.add(new DERTaggedObject(false, 1, certissuer));
        }

        if (certserno != null)
        {
            v.add(new DERTaggedObject(false, 2, certserno));
        }


        return new DERSequence(v);
    }
}

