/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org) 
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509.extensions;

import org.bouncycastle.asn1.x509.qualified.BiometricData;

import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.ASN1TaggedObject;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERSequence;

import java.util.Vector;
import java.util.Iterator;

/** Extension {@code BiometricSyntax} (RFC 3039).

   <p>Documentation from <b>RFC 3039</b>:</p>
   
   <p>This section defines an extension for storage of biometric
   information.  Biometric information is stored in the form of a hash
   of a biometric template.</p>

   <p>The purpose of this extension is to provide means for authentication
   of biometric information.  The biometric information that corresponds
   to the stored hash is not stored in this extension, but the extension
   MAY include an URI pointing to a location where this information can
   be obtained.  If included, this URI does not imply that this is the
   only way to access this information.</p>

   <p>It is RECOMMENDED that biometric information in this extension is
   limited to information types suitable for human verification, i.e.,
   where the decision of whether the information is an accurate
   representation of the subject is naturally performed by a person.
   This implies a usage where the biometric information is represented
   by, for example, a graphical image displayed to the relying party,
   which MAY be used by the relying party to enhance identification of
   the subject.</p>

   <p>This extension MUST NOT be marked critical.</p>
   <p>ASN.1 definitions:</p>
   <pre>

      biometricInfo  EXTENSION ::= {
          SYNTAX             BiometricSyntax
          IDENTIFIED BY      id-pe-biometricInfo }

      id-pe-biometricInfo OBJECT IDENTIFIER  ::= {id-pe 2}

      BiometricSyntax ::= SEQUENCE OF BiometricData

      BiometricData ::= SEQUENCE {
          typeOfBiometricData  TypeOfBiometricData,
          hashAlgorithm        AlgorithmIdentifier,
          biometricDataHash    OCTET STRING,
          sourceDataUri        IA5String OPTIONAL }

      TypeOfBiometricData ::= CHOICE {
          predefinedBiometricType    PredefinedBiometricType,
          biometricDataID            OBJECT IDENTIFIER }

      PredefinedBiometricType ::= INTEGER { picture(0),
          handwritten-signature(1)} (picture|handwritten-signature,...)
   </pre>
   <p>The predefined biometric type picture, when present, SHALL identify
   that the source picture is in the form of a displayable graphical
   image of the subject.  The hash of the graphical image SHALL only be
   calculated over the image data excluding any labels defining the
   image type.</p>

   <p>The predefined biometric type handwritten-signature, when present,
   SHALL identify that the source data is in the form of a displayable
   graphical image of the subject's handwritten signature.  The hash of
   the graphical image SHALL only be calculated over the image data
   excluding any labels defining the image type.</p>
 
 * @author crackapat
 */
public class BiometricSyntax extends ASN1Encodable{
    
    Vector<BiometricData> biometricDataVector;
    
    /** Creates a new instance from a {@code ASN1Sequence} object. */
    public BiometricSyntax(ASN1Sequence seq) {
        biometricDataVector = new Vector<BiometricData>();
        for(int i =0;i<seq.size();i++){
            biometricDataVector.add(BiometricData.getInstance(seq.getObjectAt(i)));
        }
    }
    /** Gets a new instance from a {@code ASN1Sequence} or 
     * {@code BiometricSyntax} object.*/
    public static BiometricSyntax getInstance(Object obj){
        if(obj instanceof BiometricSyntax){
            return (BiometricSyntax)obj;
        }
        if(obj instanceof ASN1Sequence){
            return new BiometricSyntax((ASN1Sequence)obj);
        }
        throw new IllegalArgumentException("invalid sequence");
    }

    /** Returns the vector of {@code biometricData}.*/
    public Vector<BiometricData> getBiometricDataVector() {
        return biometricDataVector;
    }

    /**@inheritDoc
     */
    public DERObject toASN1Object(){
        ASN1EncodableVector vec = new ASN1EncodableVector();
        Iterator<BiometricData> it = biometricDataVector.iterator();
        while(it.hasNext()){
            vec.add(it.next().toASN1Object());
        }
        return new DERSequence(vec);
    }
    
}
