/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org) 
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 */

package be.cardon.asn1.x509.extensions;

import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.ASN1TaggedObject;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERSequence;

import be.cardon.asn1.x509.DistributionPoint;

import java.util.Vector;
import java.util.Iterator;

/**Extension {@code CRLDistributionPoints}.
   <p>Documentation from <b>RFC 3280</b>:</p>
 
   <p>The CRL distribution points extension identifies how CRL information
   is obtained.  The extension SHOULD be non-critical, but this profile
   RECOMMENDS support for this extension by CAs and applications.
   Further discussion of CRL management is contained in section 5.</p>

   <p>The {@code cRLDistributionPoints} extension is a {@code SEQUENCE of}
   {@code DistributionPoint}.  A {@code DistributionPoint} consists of three fields,
   each of which is optional: {@code distributionPoint}, {@code reasons}, 
   and {@code cRLIssuer}.
   While each of these fields is optional, a {@code DistributionPoint} MUST NOT
   consist of only the {@code reasons} field; either {@code distributionPoint} or
   {@code cRLIssuer} MUST be present.  If the certificate issuer is not the CRL
   issuer, then the cRLIssuer field MUST be present and contain the Name
   of the CRL issuer.  If the certificate issuer is also the CRL issuer,
   then the {@code cRLIssuer} field MUST be omitted and the {@code distributionPoint}
   field MUST be present.  If the {@code distributionPoint} field is omitted,
   {@code cRLIssuer} MUST be present and include a {@code Name} corresponding to an
   X.500 or LDAP directory entry where the CRL is located.</p>

   <p>When the {@code distributionPoint} field is present, it contains either a
   SEQUENCE of general names or a single value, {@code nameRelativeToCRLIssuer}.
   If the {@code cRLDistributionPoints} extension contains a general name of
   type URI, the following semantics MUST be assumed: the URI is a
   pointer to the current CRL for the associated reasons and will be
   issued by the associated {@code cRLIssuer}.  The expected values for the URI
   are those defined in 4.2.1.7.  Processing rules for other values are
   not defined by this specification.</p>

   <p>If the {@code DistributionPointName} contains multiple values, each name
   describes a different mechanism to obtain the same CRL.  For example,
   the same CRL could be available for retrieval through both LDAP and
   HTTP.</p>

   <p>If the {@code DistributionPointName} contains the single value
   {@code nameRelativeToCRLIssuer}, the value provides a distinguished name
   fragment.  The fragment is appended to the X.500 distinguished name
   of the CRL issuer to obtain the distribution point name.  If the
 * {@code cRLIssuer} field in the {@code DistributionPoint} is present, then the name
   fragment is appended to the distinguished name that it contains;
   otherwise, the name fragment is appended to the certificate issuer
   distinguished name.  The {code DistributionPointName} MUST NOT use the
 * {@code nameRealtiveToCRLIssuer} alternative when {@code cRLIssuer} contains more than
   one distinguished name.</p>

   <p>If the {@code DistributionPoint} omits the {@code reasons} field, the CRL MUST
   include revocation information for all reasons.</p>

   <p>The {@code cRLIssuer} identifies the entity who signs and issues the CRL.  If
   present, the {@code cRLIssuer} MUST contain at least one an X.500
   distinguished name (DN), and MAY also contain other name forms.
   Since the {@code cRLIssuer} is compared to the CRL issuer name, the X.501
   type Name MUST follow the encoding rules for the issuer name field in
   the certificate (section 4.1.2.4).</p>

   <p>ASN.1 definitions:</p>
   <pre>
   id-ce-cRLDistributionPoints OBJECT IDENTIFIER ::=  { id-ce 31 }

   CRLDistributionPoints ::= SEQUENCE SIZE (1..MAX) OF DistributionPoint

   DistributionPoint ::= SEQUENCE {
        distributionPoint       [0]     DistributionPointName OPTIONAL,
        reasons                 [1]     ReasonFlags OPTIONAL,
        cRLIssuer               [2]     GeneralNames OPTIONAL }

   DistributionPointName ::= CHOICE {
        fullName                [0]     GeneralNames,
        nameRelativeToCRLIssuer [1]     RelativeDistinguishedName }

   ReasonFlags ::= BIT STRING {
        unused                  (0),
        keyCompromise           (1),
        cACompromise            (2),
        affiliationChanged      (3),
        superseded              (4),
        cessationOfOperation    (5),
        certificateHold         (6),
        privilegeWithdrawn      (7),
        aACompromise            (8) }
   </pre>
 */
public class CRLDistributionPoints
    extends ASN1Encodable
{
    Vector<DistributionPoint> distributionPointVector;

    /**Returns an instance from a {@literal ASN.1} tagged object.*/
    public static CRLDistributionPoints getInstance(
        ASN1TaggedObject obj,
        boolean          explicit)
    {
        return getInstance(ASN1Sequence.getInstance(obj, explicit));
    }
    
    /**Returns an instance from a {@code ASN1Sequence}, or
     *{@code CRLDistributionPoints} object.*/
    public static CRLDistributionPoints getInstance(
        Object  obj)
    {
        if (obj instanceof CRLDistributionPoints)
        {
            return (CRLDistributionPoints)obj;
        }
        else if (obj instanceof ASN1Sequence)
        {
            return new CRLDistributionPoints((ASN1Sequence)obj);
        }

        throw new IllegalArgumentException("unknown object in factory");
    }

    /**Creates an new instance from a {@code ASN1Sequence} object.*/
    public CRLDistributionPoints(
        ASN1Sequence seq)
    {
        distributionPointVector = new Vector<DistributionPoint>();
        for (int i = 0; i != seq.size(); i++)
        {
            distributionPointVector.add(DistributionPoint.getInstance(seq.getObjectAt(i)));
        }
    }
    /**Create an new instance from given distribution points.*/
    public CRLDistributionPoints(
        Vector<DistributionPoint> distributionPointVector)
    {
        this.distributionPointVector = distributionPointVector;
    }

    /**
     * Returns the distribution points making up the sequence.
     */
    public Vector<DistributionPoint> getDistributionPointVector() {
        return distributionPointVector;
    }

    /**
     *@inheritDoc
     */
    public DERObject toASN1Object()
    {
        ASN1EncodableVector  v = new ASN1EncodableVector();
        Iterator<DistributionPoint> it = distributionPointVector.iterator();
        while(it.hasNext()){
             v.add(it.next().toASN1Object());
        }
        return new DERSequence(v);
    }
}