/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org) 
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509.extensions;


import be.cardon.asn1.x509.*;
import java.util.Enumeration;
import java.util.Vector;

import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.ASN1TaggedObject;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERSequence;

/**
 * Extension {@code ExtendedKeyUsage}.
  
   <p>Documentation from <b>RFC 3280</b>:</p>
   <p>This extension indicates one or more purposes for which the certified
   public key may be used, in addition to or in place of the basic
   purposes indicated in the key usage extension.  In general, this
   extension will appear only in end entity certificates.  This
   extension is defined as follows:</p>
   <pre>
   id-ce-extKeyUsage OBJECT IDENTIFIER ::= { id-ce 37 }
   
   ExtKeyUsageSyntax ::= SEQUENCE SIZE (1..MAX) OF KeyPurposeId

   KeyPurposeId ::= OBJECT IDENTIFIER
   </pre>
   <p>Key purposes may be defined by any organization with a need.  Object
   identifiers used to identify key purposes MUST be assigned in
   accordance with IANA or ITU-T Recommendation X.660 [X.660].</p>

   <p>This extension MAY, at the option of the certificate issuer, be
   either critical or non-critical.</p>

   <p>If the extension is present, then the certificate MUST only be used
   for one of the purposes indicated.  If multiple purposes are
   indicated the application need not recognize all purposes indicated,
   as long as the intended purpose is present.  Certificate using
   applications MAY require that a particular purpose be indicated in
   order for the certificate to be acceptable to that application.</p>

   <p>If a CA includes extended key usages to satisfy such applications,
   but does not wish to restrict usages of the key, the CA can include
   the special keyPurposeID anyExtendedKeyUsage.  If the
   anyExtendedKeyUsage keyPurposeID is present, the extension SHOULD NOT
   be critical.</p>

   <p>If a certificate contains both a key usage extension and an extended
   key usage extension, then both extensions MUST be processed
   independently and the certificate MUST only be used for a purpose
   consistent with both extensions.  If there is no purpose consistent
   with both extensions, then the certificate MUST NOT be used for any
   purpose.</p>

   <p>The following key usage purposes are defined:</p>
   <pre>
   anyExtendedKeyUsage OBJECT IDENTIFIER ::= { id-ce-extKeyUsage 0 }

   id-kp OBJECT IDENTIFIER ::= { id-pkix 3 }

   id-kp-serverAuth             OBJECT IDENTIFIER ::= { id-kp 1 }
   -- TLS WWW server authentication
   -- Key usage bits that may be consistent: digitalSignature,
   -- keyEncipherment or keyAgreement

   id-kp-clientAuth             OBJECT IDENTIFIER ::= { id-kp 2 }
   -- TLS WWW client authentication
   -- Key usage bits that may be consistent: digitalSignature
   -- and/or keyAgreement

   id-kp-codeSigning             OBJECT IDENTIFIER ::= { id-kp 3 }
   -- Signing of downloadable executable code
   -- Key usage bits that may be consistent: digitalSignature

   id-kp-emailProtection         OBJECT IDENTIFIER ::= { id-kp 4 }
   -- E-mail protection
   -- Key usage bits that may be consistent: digitalSignature,
   -- nonRepudiation, and/or (keyEncipherment or keyAgreement)

   id-kp-timeStamping            OBJECT IDENTIFIER ::= { id-kp 8 }
   -- Binding the hash of an object to a time
   -- Key usage bits that may be consistent: digitalSignature
   -- and/or nonRepudiation

   id-kp-OCSPSigning            OBJECT IDENTIFIER ::= { id-kp 9 }
   -- Signing OCSP responses
   -- Key usage bits that may be consistent: digitalSignature
   -- and/or nonRepudiation
   </pre>
 */
public class ExtendedKeyUsage
        extends ASN1Encodable
{
    
     /** Usage {@code serverAuth}.*/
     public static final DERObjectIdentifier serverAuth 
             = new DERObjectIdentifier("1.3.6.1.5.5.7.3.1");
     
     /** Usage {@code clientAuth}.*/
     public static final DERObjectIdentifier clientAuth
             = new DERObjectIdentifier("1.3.6.1.5.5.7.3.2");
     
     /** Usage {@code codeSigning}.*/
     public static final DERObjectIdentifier codeSigning
             = new DERObjectIdentifier("1.3.6.1.5.5.7.3.3");
     
     /** Usage {@code emailProtection}.*/
     public static final DERObjectIdentifier emailProtection
             = new DERObjectIdentifier("1.3.6.1.5.5.7.3.4");     
     
     /** Usage {@code timeStamping}.*/
     public static final DERObjectIdentifier timeStamping
             = new DERObjectIdentifier("1.3.6.1.5.5.7.3.8");
     
     /** Usage {@code OCSPSigning}.*/
     public static final DERObjectIdentifier OCSPSigning
             = new DERObjectIdentifier("1.3.6.1.5.5.7.3.9");
    
    
    Vector<DERObjectIdentifier> keyPurposeIds = new Vector<DERObjectIdentifier>();

    /**Returns an instance from a {@literal ASN.1}tagged object.*/
    public static ExtendedKeyUsage getInstance(
        ASN1TaggedObject obj,
        boolean          explicit)
    {
        return getInstance(ASN1Sequence.getInstance(obj, explicit));
    }

    /**Returns an instance from a {@code ASN1Sequence}, 
     * or a {@code ExtendedKeyUsage} object.*/
    public static ExtendedKeyUsage getInstance(
        Object obj)
    {
        if(obj == null || obj instanceof ExtendedKeyUsage) 
        {
            return (ExtendedKeyUsage)obj;
        }
        
        if(obj instanceof ASN1Sequence) 
        {
            return new ExtendedKeyUsage((ASN1Sequence)obj);
        }
        
        throw new IllegalArgumentException("Invalid ExtendedKeyUsage: " + obj.getClass().getName());
    }

    /**Returns the {@code keyPurposeIds} value.*/
    public Vector<DERObjectIdentifier> getKeyPurposeIds() {
        return keyPurposeIds;
    }

   /**Creates a new instance from a {@code ASN1Sequence} object.*/
    public ExtendedKeyUsage(
        ASN1Sequence  seq)
    {
        Enumeration e = seq.getObjects();
        while (e.hasMoreElements())
        {
            DERObjectIdentifier oid = DERObjectIdentifier.getInstance(e.nextElement());
            keyPurposeIds.add(oid);
        }
    }

    /**Creates a new instance with the given {@code keyPurposeIds}.*/
    public ExtendedKeyUsage(Vector<DERObjectIdentifier> keyPurposeIds)
    {
            this.keyPurposeIds = keyPurposeIds;
    }

    /**@inheritDoc
     */
    public DERObject toASN1Object()
    {
        ASN1EncodableVector v = new ASN1EncodableVector();
        Enumeration<DERObjectIdentifier>e = keyPurposeIds.elements();
        while (e.hasMoreElements()){
            v.add(e.nextElement());
        }
        return new DERSequence(v);
    }
}
