/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org) 
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509.extensions;

import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.ASN1TaggedObject;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERSequence;

import be.cardon.asn1.x509.DistributionPoint;

import java.util.Vector;
import java.util.Iterator;

/**Extension {@code FreshestCRL}.
   <p>Documentation from <b>FRC 3280</b>:</p>
  
   <p>The freshest CRL extension identifies how delta CRL information is
   obtained.  The extension MUST be non-critical.  Further discussion of
   CRL management is contained in section 5.</p>

   <p>The same syntax is used for this extension and the
   cRLDistributionPoints extension, and is described in section
   4.2.1.14.  The same conventions apply to both extensions.</p>

   <p>ASN.1 Definition</p>
   <pre>
   id-ce-freshestCRL OBJECT IDENTIFIER ::=  { id-ce 46 }

   FreshestCRL ::= CRLDistributionPoints
   </pre>
 *@see CRLDistributionPoints
 * @author crackapat
 */
public class FreshestCRL
    extends ASN1Encodable
{
    Vector<DistributionPoint> distributionPointVector;

    /**Returns an instance from a {@literal ASN.1} tagged object.*/
    public static FreshestCRL getInstance(
        ASN1TaggedObject obj,
        boolean          explicit)
    {
        return getInstance(ASN1Sequence.getInstance(obj, explicit));
    }
    
    /**Returns an instance from a {@code ASN1Sequence}, or
     *{@code FreshestCRL} object.*/
    public static FreshestCRL getInstance(
        Object  obj)
    {
        if (obj instanceof FreshestCRL)
        {
            return (FreshestCRL)obj;
        }
        else if (obj instanceof ASN1Sequence)
        {
            return new FreshestCRL((ASN1Sequence)obj);
        }

        throw new IllegalArgumentException("unknown object in factory");
    }

    /**Creates an new instance from a {@code ASN1Sequence} object.*/
    public FreshestCRL(
        ASN1Sequence seq)
    {
        distributionPointVector = new Vector<DistributionPoint>();
        for (int i = 0; i != seq.size(); i++)
        {
            distributionPointVector.add(DistributionPoint.getInstance(seq.getObjectAt(i)));
        }
    }
    /**Create an new instance from given distribution points.*/
    public FreshestCRL(
        Vector<DistributionPoint> distributionPointVector)
    {
        this.distributionPointVector = distributionPointVector;
    }

    /**
     * Returns the distribution points making up the sequence.
     */
    public Vector<DistributionPoint> getDistributionPointVector() {
        return distributionPointVector;
    }

    /**
     *@inheritDoc
     */
    public DERObject toASN1Object()
    {
        ASN1EncodableVector  v = new ASN1EncodableVector();
        Iterator<DistributionPoint> it = distributionPointVector.iterator();
        while(it.hasNext()){
             v.add(it.next().toASN1Object());
        }
        return new DERSequence(v);
    }
}