/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org)  
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */
package be.cardon.asn1.x509.extensions;

import be.cardon.utils.ByteArrayTools;
import org.bouncycastle.asn1.DERBitString;
import org.bouncycastle.asn1.DEREncodable;

/**
   Extension {@code KeyUsage}.
 
   <p>Documentation from <b>RFC 3280</b>:</p>

   <p>The key usage extension defines the purpose (e.g., encipherment,
   signature, certificate signing) of the key contained in the
   certificate.  The usage restriction might be employed when a key that
   could be used for more than one operation is to be restricted.  For
   example, when an RSA key should be used only to verify signatures on
   objects other than public key certificates and CRLs, the
   digitalSignature and/or nonRepudiation bits would be asserted.
   Likewise, when an RSA key should be used only for key management, the
   keyEncipherment bit would be asserted.</p>

   <p>This extension MUST appear in certificates that contain public keys
   that are used to validate digital signatures on other public key
   certificates or CRLs.  When this extension appears, it SHOULD be
   marked critical.</p>
 
   <p>ASN.1 definitions: </p>
   <pre>
      id-ce-keyUsage OBJECT IDENTIFIER ::=  { id-ce 15 }

      KeyUsage ::= BIT STRING {
           digitalSignature        (0),
           nonRepudiation          (1),
           keyEncipherment         (2),
           dataEncipherment        (3),
           keyAgreement            (4),
           keyCertSign             (5),
           cRLSign                 (6),
           encipherOnly            (7),
           decipherOnly            (8) }
   </pre>
   <p>Bits in the {@code KeyUsage} type are used as follows:</p>
   <ul>
      <li>The {@code digitalSignature} bit is asserted when the subject public key
      is used with a digital signature mechanism to support security
      services other than certificate signing (bit 5), or CRL signing
      (bit 6).  Digital signature mechanisms are often used for entity
      authentication and data origin authentication with integrity.</li>

      <li>The {@code nonRepudiation} bit is asserted when the subject public key is
      used to verify digital signatures used to provide a non-
      repudiation service which protects against the signing entity
      falsely denying some action, excluding certificate or CRL signing.
      In the case of later conflict, a reliable third party may
      determine the authenticity of the signed data.

      Further distinctions between the{@code digitalSignature} and
      {@code nonRepudiation} bits may be provided in specific certificate
      policies.</li>

      <li>The {@code keyEncipherment} bit is asserted when the subject public key is
      used for key transport.  For example, when an RSA key is to be
      used for key management, then this bit is set.</li>

      <li>The {@code dataEncipherment} bit is asserted when the subject public key
      is used for enciphering user data, other than cryptographic keys.</li>

      <li>The {@code keyAgreement} bit is asserted when the subject public key is
      used for key agreement.  For example, when a Diffie-Hellman key is
      to be used for key management, then this bit is set.</li>

      <li>The {@code keyCertSign} bit is asserted when the subject public key is
      used for verifying a signature on public key certificates.  If the
      {@code keyCertSign} bit is asserted, then the cA bit in the basic
      constraints extension (section 4.2.1.10) MUST also be asserted.</li>

      <li>The {@code cRLSign} bit is asserted when the subject public key is used
      for verifying a signature on certificate revocation list (e.g., a
      CRL, delta CRL, or an ARL).  This bit MUST be asserted in
      certificates that are used to verify signatures on CRLs.</li>

      <li>The meaning of the {@code encipherOnly} bit is undefined in the absence of
      the {@code keyAgreement} bit.  When the encipherOnly bit is asserted and
      the {@code keyAgreement} bit is also set, the subject public key may be
      used only for enciphering data while performing key agreement.</li>

      <li>The meaning of the {@code decipherOnly} bit is undefined in the absence of
      the {@code keyAgreement} bit.  When the decipherOnly bit is asserted and
      the {@code keyAgreement} bit is also set, the subject public key may be
      used only for deciphering data while performing key agreement.</li>
   </ul>
   <p>This profile does not restrict the combinations of bits that may be
   set in an instantiation of the keyUsage extension.  However,
   appropriate values for keyUsage extensions for particular algorithms
   are specified in [PKIXALGS].</p>

 */
public class KeyUsage
    extends DERBitString{
    

    /** Usage {@code digitalSignature}*/
    public static final int        digitalSignature = (1 << 7); 
    /** Usage {@code nonRepudiation}*/
    public static final int        nonRepudiation   = (1 << 6);
    /** Usage {@code keyEncipherment}*/
    public static final int        keyEncipherment  = (1 << 5);
    /** Usage {@code dataEncipherment}*/
    public static final int        dataEncipherment = (1 << 4);
    /** Usage {@code keyAgreement}*/
    public static final int        keyAgreement     = (1 << 3);
    /** Usage {@code keyCertSign}*/
    public static final int        keyCertSign      = (1 << 2);
    /** Usage {@code cRLSign}*/
    public static final int        cRLSign          = (1 << 1);
    /** Usage {@code encipherOnly}*/
    public static final int        encipherOnly     = (1 << 0);
    /** Usage {@code decipherOnly}*/
    public static final int        decipherOnly     = (1 << 15);

    /**
     * Creates an new object with the given usage(s). 
     * 
     * @param usage - the bitwise {@code OR} of the Key Usage flags giving the
     * allowed uses for the key.
     * e.g. {@code (KeyUsage.keyEncipherment | KeyUsage.dataEncipherment)}
     */
    public KeyUsage(
        int usage)
    {
        super(DERBitString.getBytes(usage), DERBitString.getPadBits(usage));
    }
    /**
     * Creates an new object from an existing ASN.1 {@code DERBitString KeyUsage}. 
     */
    public KeyUsage(
        DERBitString usage)
    { 
        super(usage.getBytes(), usage.getPadBits());
    }
  
    /**Returns the {@code keyUsage} value.*/
    public int getKeyUsage(){
        return super.intValue();
    }
}
