/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org) 
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509.extensions;

import be.cardon.asn1.x509.*;
import java.util.Enumeration;
import java.util.Vector;

import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.ASN1TaggedObject;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERSequence;
import org.bouncycastle.asn1.DERTaggedObject;

 /**
   Extension {@code NameConstraints}.

   <p>Documentation from <b>RFC 3280</b>:</p>
   <p>The name constraints extension, which MUST be used only in a CA
   certificate, indicates a name space within which all subject names in
   subsequent certificates in a certification path MUST be located.
   Restrictions apply to the subject distinguished name and apply to
   subject alternative names.  Restrictions apply only when the
   specified name form is present.  If no name of the type is in the
   certificate, the certificate is acceptable.<p>

   <p>Name constraints are not applied to certificates whose issuer and
   subject are identical (unless the certificate is the final
   certificate in the path).  (This could prevent CAs that use name
   constraints from employing self-issued certificates to implement key
   rollover.)</p>

   <p>Restrictions are defined in terms of permitted or excluded name
   subtrees.  Any name matching a restriction in the excludedSubtrees
   field is invalid regardless of information appearing in the
   permittedSubtrees.  This extension MUST be critical.</p>

   <p>Within this profile, the minimum and maximum fields are not used with
   any name forms, thus minimum MUST be zero, and maximum MUST be
   absent.</p>

   <p>For URIs, the constraint applies to the host part of the name.  The
   constraint MAY specify a host or a domain.  Examples would be
   "foo.bar.com";  and ".xyz.com".  When the the constraint begins with
   a period, it MAY be expanded with one or more subdomains.  That is,
   the constraint ".xyz.com" is satisfied by both abc.xyz.com and
   abc.def.xyz.com.  However, the constraint ".xyz.com" is not satisfied
   by "xyz.com".  When the constraint does not begin with a period, it
   specifies a host.</p>

   <p>A name constraint for Internet mail addresses MAY specify a
   particular mailbox, all addresses at a particular host, or all
   mailboxes in a domain.  To indicate a particular mailbox, the
   constraint is the complete mail address.  For example, "root@xyz.com"
   indicates the root mailbox on the host "xyz.com".  To indicate all
   Internet mail addresses on a particular host, the constraint is
   specified as the host name.  For example, the constraint "xyz.com" is
   satisfied by any mail address at the host "xyz.com".  To specify any
   address within a domain, the constraint is specified with a leading
   period (as with URIs).  For example, ".xyz.com" indicates all the
   Internet mail addresses in the domain "xyz.com", but not Internet
   mail addresses on the host "xyz.com".</p>

   <p>DNS name restrictions are expressed as foo.bar.com.  Any DNS name
   that can be constructed by simply adding to the left hand side of the
   name satisfies the name constraint.  For example, www.foo.bar.com
   would satisfy the constraint but foo1.bar.com would not.</p>

   <p>Legacy implementations exist where an RFC 822 name is embedded in the
   subject distinguished name in an attribute of type EmailAddress
   (section 4.1.2.6).  When rfc822 names are constrained, but the
   certificate does not include a subject alternative name, the rfc822
   name constraint MUST be applied to the attribute of type EmailAddress
   in the subject distinguished name.  The ASN.1 syntax for EmailAddress
   and the corresponding OID are supplied in Appendix A.<P>

   <p>Restrictions of the form directoryName MUST be applied to the subject
   field in the certificate and to the subjectAltName extensions of type
   directoryName.  Restrictions of the form x400Address MUST be applied
   to subjectAltName extensions of type x400Address.</p>

   <p>When applying restrictions of the form directoryName, an
   implementation MUST compare DN attributes.  At a minimum,
   implementations MUST perform the DN comparison rules specified in
   Section 4.1.2.4.  CAs issuing certificates with a restriction of the
   form directoryName SHOULD NOT rely on implementation of the full ISO
   DN name comparison algorithm.  This implies name restrictions MUST be
   stated identically to the encoding used in the subject field or
   subjectAltName extension.</p>

   <p>The syntax of iPAddress MUST be as described in section 4.2.1.7 with
   the following additions specifically for Name Constraints.  For IPv4
   addresses, the ipAddress field of generalName MUST contain eight (8)
   octets, encoded in the style of RFC 1519 (CIDR) to represent an
   address range [RFC 1519].  For IPv6 addresses, the ipAddress field
   MUST contain 32 octets similarly encoded.  For example, a name
   constraint for "class C" subnet 10.9.8.0 is represented as the octets
   0A 09 08 00 FF FF FF 00, representing the CIDR notation
   10.9.8.0/255.255.255.0.</p>

   <p>The syntax and semantics for name constraints for otherName,
   ediPartyName, and registeredID are not defined by this specification.</p>
  
   <p>ASN.1 definitions:</p>
   <pre>
      id-ce-nameConstraints OBJECT IDENTIFIER ::=  { id-ce 30 }

      NameConstraints ::= SEQUENCE {
           permittedSubtrees       [0]     GeneralSubtrees OPTIONAL,
           excludedSubtrees        [1]     GeneralSubtrees OPTIONAL }

      GeneralSubtrees ::= SEQUENCE SIZE (1..MAX) OF GeneralSubtree

      GeneralSubtree ::= SEQUENCE {
           base                    GeneralName,
           minimum         [0]     BaseDistance DEFAULT 0,
           maximum         [1]     BaseDistance OPTIONAL }

      BaseDistance ::= INTEGER (0..MAX)
   </pre>
     */
public class NameConstraints
    extends ASN1Encodable
{
    private GeneralSubtrees permittedSubtrees;
    private GeneralSubtrees excludedSubtrees;

    /**Returns an instance from a {@code ASN1Sequence} 
     * or a {@code NameConstraints} object.*/
    public static NameConstraints getInstance(Object obj){
        if(obj instanceof NameConstraints){
            return (NameConstraints)obj;
        }
        if(obj instanceof ASN1Sequence){
            return new NameConstraints((ASN1Sequence)obj);
        }
        throw new IllegalArgumentException("Invalid sequence");
    }
    /**Creates an new instance from a {@code ASN1Sequence} object.*/
    public NameConstraints(ASN1Sequence seq)
    {
        Enumeration e = seq.getObjects();
        while (e.hasMoreElements())
        {
            ASN1TaggedObject o = ASN1TaggedObject.getInstance(e.nextElement());
            switch (o.getTagNo())
            {
            case 0:
                permittedSubtrees = GeneralSubtrees.getInstance(
                        ASN1Sequence.getInstance(o, false));
                break;
            case 1:
                excludedSubtrees = GeneralSubtrees.getInstance(
                        ASN1Sequence.getInstance(o, false));
                break;
            }
        }
    }

    /**Creates an new instance from the given {@code permittedSubtrees} and 
     * {@code excludedSubtrees}.*/
     public NameConstraints(
        GeneralSubtrees permitted,
        GeneralSubtrees excluded)
    {
        this.permittedSubtrees = permitted;
        this.excludedSubtrees = excluded;
    }

    /**Returns the {@code excludedSubtrees} value, or {@code null} is absent.*/
    public GeneralSubtrees getExcludedSubtrees() {
        return excludedSubtrees;
    }
    /**Returns the {@code permittedSubtrees} value, or {@code null} is absent.*/
    public GeneralSubtrees getPermittedSubtrees() {
        return permittedSubtrees;
    }

    /**@inheritDoc
     */
    public DERObject toASN1Object() 
    {
        ASN1EncodableVector v = new ASN1EncodableVector();

        if (permittedSubtrees != null) 
        {
            v.add(new DERTaggedObject(false, 0, permittedSubtrees.toASN1Object()));
        }

        if (excludedSubtrees != null) 
        {
            v.add(new DERTaggedObject(false, 1, excludedSubtrees.toASN1Object()));
        }

        return new DERSequence(v);
    }
}

