/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org)  
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509.extensions;


import be.cardon.asn1.x509.*;
import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.ASN1TaggedObject;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERSequence;
import org.bouncycastle.asn1.DERInteger;
import java.math.BigInteger;
import org.bouncycastle.asn1.DERTaggedObject;

/**Extension {@code PolicyConstraints}.
 
   <p>Documentation from <b>FRC 3280</b>:</p>
   <p>The policy constraints extension can be used in certificates issued
   to CAs.  The policy constraints extension constrains path validation
   in two ways.  It can be used to prohibit policy mapping or require
   that each certificate in a path contain an acceptable policy
   identifier.</p>

   <p>If the inhibitPolicyMapping field is present, the value indicates the
   number of additional certificates that may appear in the path before
   policy mapping is no longer permitted.  For example, a value of one
   indicates that policy mapping may be processed in certificates issued
   by the subject of this certificate, but not in additional
   certificates in the path.</p>

   <p>If the requireExplicitPolicy field is present, the value of
   requireExplicitPolicy indicates the number of additional certificates
   that may appear in the path before an explicit policy is required for
   the entire path.  When an explicit policy is required, it is
   necessary for all certificates in the path to contain an acceptable
   policy identifier in the certificate policies extension.  An
   acceptable policy identifier is the identifier of a policy required
   by the user of the certification path or the identifier of a policy
   which has been declared equivalent through policy mapping.</p>

   <p>Conforming CAs MUST NOT issue certificates where policy constraints
   is a empty sequence.  That is, at least one of the
   inhibitPolicyMapping field or the requireExplicitPolicy field MUST be
   present.  The behavior of clients that encounter a empty policy
   constraints field is not addressed in this profile.</p>

   <p>This extension MAY be critical or non-critical.</p>
   
   <p>ASN.1 definitions:</p>
   <pre>
   id-ce-policyConstraints OBJECT IDENTIFIER ::=  { id-ce 36 }

   PolicyConstraints ::= SEQUENCE {
        requireExplicitPolicy           [0] SkipCerts OPTIONAL,
        inhibitPolicyMapping            [1] SkipCerts OPTIONAL }

   SkipCerts ::= INTEGER (0..MAX)
   </pre>
 *
 * @author crackapat
 */
public class PolicyConstraints extends ASN1Encodable{
    
    int requireExplicitPolicy = -1;
    int inhibitPolicyMapping = -1;
    
    /**Returns an instance from {@code ASN1Sequence}, or
     *{@code PolicyConstraints} object.*/
    public static PolicyConstraints getInstance(Object obj){
        if(obj instanceof PolicyConstraints){
            return (PolicyConstraints)obj;
        }
        if(obj instanceof ASN1Sequence){
            return new PolicyConstraints((ASN1Sequence)obj);
        }
        throw new IllegalArgumentException("invalid sequence");
    }
    
    /**Creates an new instance from {@code ASN1Sequence} object.*/
    public PolicyConstraints(ASN1Sequence seq){
        if(seq.size()>2){
            throw new IllegalArgumentException("sequence > 2");
        }
        for(int i=0;i<seq.size();i++){
            ASN1TaggedObject taggedObj = ASN1TaggedObject.getInstance(seq.getObjectAt(i));
            switch(taggedObj.getTagNo()){
                case 0:
                    requireExplicitPolicy = DERInteger.getInstance(taggedObj.getObject()).getValue().intValue();
                case 1:
                    inhibitPolicyMapping =  DERInteger.getInstance(taggedObj.getObject()).getValue().intValue();
                default:
                    throw new IllegalArgumentException("wrong tag number");
            }
        }
        
    }
    
    /** Creates a new instance with the given {@code requireExplicitPolicy} and
     * {@code inhibitPolicyMapping}.
     */
    public PolicyConstraints(int requireExplicitPolicy, int inhibitPolicyMapping) {
        this.requireExplicitPolicy = requireExplicitPolicy;
        this.inhibitPolicyMapping = inhibitPolicyMapping;
    }

    /** Returns the {@code requireExplicitPolicy} or {@code null} is absent.*/
    public int getRequireExplicitPolicy() {
        return requireExplicitPolicy;
    }
    /** Returns the {@code inhibitPolicyMapping} or {@code null} is absent.*/
    public int getInhibitPolicyMapping() {
        return inhibitPolicyMapping;
    }
    /**@inheritDoc
     */
    public DERObject toASN1Object(){
        ASN1EncodableVector vec = new ASN1EncodableVector();
        if(requireExplicitPolicy != -1){
            vec.add(new DERTaggedObject(0,
                new DERInteger(requireExplicitPolicy)));
        }
        if(inhibitPolicyMapping != -1){
            vec.add(new DERTaggedObject(1,
                new DERInteger(inhibitPolicyMapping)));            
        }
        return new DERSequence(vec);
        
    }
}
