/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org) 
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509.extensions;


import be.cardon.asn1.x509.*;
import java.util.Hashtable;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Enumeration;
import java.util.Iterator;

import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERSequence;

/**
   Extension {@code PolicyMappings}.
 
   <p>Documentation from <b>RFC 3280</b>:</p>
 
   <p>This extension is used in CA certificates.  It lists one or more
   pairs of OIDs; each pair includes an {@code issuerDomainPolicy} and a
   {@code subjectDomainPolicy}.  The pairing indicates the issuing CA considers
   its {@code issuerDomainPolicy} equivalent to the subject CA's
   {@code subjectDomainPolicy}.</p>

   <p>The issuing CA's users might accept an {@code issuerDomainPolicy} for certain
   applications.  The policy mapping defines the list of policies
   associated with the subject CA that may be accepted as comparable to
   the {@code issuerDomainPolicy}.</p>

   <p>Each {@code issuerDomainPolicy} named in the policy mapping extension SHOULD
   also be asserted in a certificate policies extension in the same
   certificate.  Policies SHOULD NOT be mapped either to or from the
   special value {@code anyPolicy} (section 4.2.1.5).</p>

   <p>This extension MAY be supported by CAs and/or applications, and it
   MUST be non-critical.</p>

   <p>ASN.1 definition:</p>
   <pre>
   id-ce-policyMappings OBJECT IDENTIFIER ::=  { id-ce 33 }

   PolicyMappings ::= SEQUENCE SIZE (1..MAX) OF SEQUENCE {
        issuerDomainPolicy      CertPolicyId,
        subjectDomainPolicy     CertPolicyId }
  
   CertPolicyId ::= OBJECT IDENTIFIER

   </pre>

 */
public class PolicyMappings
    extends ASN1Encodable
{
   Map<DERObjectIdentifier, DERObjectIdentifier> map;
    
   /**Returns a new instance from an {@code ASN1Sequence} or a {@code PolicyMappings}
    *object.*/
   public static PolicyMappings getInstance(Object obj){
       if(obj instanceof PolicyMappings){
           return (PolicyMappings)obj;
       }
       if(obj instanceof ASN1Sequence){
           return new PolicyMappings((ASN1Sequence)obj);
       }
       throw new IllegalArgumentException("invalid object");
   }
   /**
    * Creates a new {@code PolicyMappings} instance from an {@code ASN1Sequence} .
    */
   public PolicyMappings (ASN1Sequence seq) 
      {
         map = new Hashtable<DERObjectIdentifier, DERObjectIdentifier>();
         for(int i=0;i<seq.size();i++){
             ASN1Sequence seq2 = DERSequence.getInstance(seq.getObjectAt(i));
             if(seq2.size()!=2){
                 throw new IllegalArgumentException("unvalid sequence.");
             }
             DERObjectIdentifier key = DERObjectIdentifier.getInstance(seq2.getObjectAt(0));
             DERObjectIdentifier value = DERObjectIdentifier.getInstance(seq2.getObjectAt(1));
             
         }
      }

   /**Returns the policy map.*/
    public Map<DERObjectIdentifier, DERObjectIdentifier> getMap() {
        return map;
    }

   /**
    * Creates a new <code>PolicyMappings</code> instance.
    */
   public PolicyMappings (Map<DERObjectIdentifier, DERObjectIdentifier> mappings) 
      {
        this.map = mappings;
      }

   public DERObject toASN1Object() 
      {
         ASN1EncodableVector dev = new ASN1EncodableVector();
         Iterator<Entry<DERObjectIdentifier,DERObjectIdentifier>> it = map.entrySet().iterator();

         while (it.hasNext())
         {
            Entry<DERObjectIdentifier,DERObjectIdentifier> entry = it.next();
            ASN1EncodableVector dv = new ASN1EncodableVector();
            dv.add(entry.getKey());
            dv.add(entry.getValue());
            dev.add(new DERSequence(dv));
      }
      return new DERSequence(dev);
   }
}

