/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org)
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509.extensions;

import be.cardon.asn1.x509.*;
import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.x509.X509Extensions;
import org.bouncycastle.asn1.x509.X509Name;
import org.bouncycastle.asn1.x509.X509ObjectIdentifiers;

import java.util.Date;

/**Extension {@code PrivateKeyUsagePeriod}. 
   <p>Documentation from <b>RFC 3280</b>:</p>
   <p>This extension SHOULD NOT be used within the Internet PKI.  CAs
   conforming to this profile MUST NOT generate certificates that
   include a critical private key usage period extension.</p>

   <p>The private key usage period extension allows the certificate issuer
   to specify a different validity period for the private key than the
   certificate.  This extension is intended for use with digital
   signature keys.  This extension consists of two optional components,
   notBefore and notAfter.  The private key associated with the
   certificate SHOULD NOT be used to sign objects before or after the
   times specified by the two components, respectively.  CAs conforming
   to this profile MUST NOT generate certificates with private key usage
   period extensions unless at least one of the two components is
   present and the extension is non-critical.</p>

   <p>Where used, {@code notBefore} and {@code notAfter} are represented 
   as {@code GeneralizedTime}
   and MUST be specified and interpreted as defined in section
   4.1.2.5.2.</p>
 
   <p>ASN.1 definitions:</p>
   <pre>
   id-ce-privateKeyUsagePeriod OBJECT IDENTIFIER ::=  { id-ce 16 }

   PrivateKeyUsagePeriod ::= SEQUENCE {
        notBefore       [0]     GeneralizedTime OPTIONAL,
        notAfter        [1]     GeneralizedTime OPTIONAL }
   </pre>
 */
public class PrivateKeyUsagePeriod 
    extends ASN1Encodable{
    
    /** {@code PrivateKeyUsagePeriod} {@literal ASN.1} Object Identifier.*/
    public final static DERObjectIdentifier OID = X509Extensions.PrivateKeyUsagePeriod;
    
    private DERGeneralizedTime notBefore=null;
    private DERGeneralizedTime notAfter=null;

    /** Creates a new instance with {@code Date} objects. */
    public PrivateKeyUsagePeriod(Date notBeforeDate, Date notAfterDate){
        if(notBefore!=null){
            this.notBefore = new DERGeneralizedTime(notBeforeDate);
        }
        if(notAfter!=null){
            this.notAfter = new DERGeneralizedTime(notAfterDate);
        }
    }
    
    /** Creates a new instance with {@literal ASN.1} {@code DERGeneralizedTime} objects. */
    public PrivateKeyUsagePeriod(DERGeneralizedTime notBefore, DERGeneralizedTime notAfter){
        this.notBefore = notBefore;
        this.notAfter = notAfter;
    }
    
    /** Creates a new instance from an {@code ASN1Sequence}. */
    public PrivateKeyUsagePeriod(ASN1Sequence seq){
        for(int i=0;i<seq.size();i++){
            DERObject derObj= seq.getObjectAt(i).getDERObject();
            ASN1TaggedObject tagObj = ASN1TaggedObject.getInstance(derObj);
            if(tagObj.getTagNo()==0){
                this.notBefore = DERGeneralizedTime.getInstance(tagObj.getObject());
            }else if(tagObj.getTagNo()==1){
                this.notAfter = DERGeneralizedTime.getInstance(tagObj.getObject());
            }else{
                throw new IllegalArgumentException("Illegal Tag > 1");
            }
        }
    }
    /** Creates a new instance from an {@code ASN1Sequence} or 
     * {@code PrivateKeyUsagePeriod} object. */
    public static PrivateKeyUsagePeriod getInstance(
        Object  obj)
    {
        if (obj instanceof PrivateKeyUsagePeriod)
        {
            return (PrivateKeyUsagePeriod)obj;
        }
        else if (obj instanceof ASN1Sequence)
        {
            return new PrivateKeyUsagePeriod((ASN1Sequence)obj);
        }

        throw new IllegalArgumentException("unknown object in factory");
    }
    
    /** Returns {@code notBefore} {@literal ASN.1} Object, or {@code null} if absent.*/
    public DERGeneralizedTime getNoteBefore(){
        return notBefore;
    }
    /** Returns {@code notBefore Date} Object, or {@code null} if absent.*/
    public Date getNotBeforeDate() throws java.text.ParseException{
        if(notBefore == null){return null;}
        return notBefore.getDate();
    }
    
    /** Returns {@code notAfter} {@literal ASN.1} Object or {@code null} if absent.*/
    public DERGeneralizedTime getNotAfter(){
        return notAfter;
    }
    
    /** Returns {@code notAfter Date} Object , or {@code null} if absent.*/
    public Date getNotAfterDate() throws java.text.ParseException{
        if(notAfter == null){return null;}
        return notAfter.getDate();
    }
    
    public DERObject toASN1Object()
    {
        ASN1EncodableVector privateKeyUsagePeriod  = new ASN1EncodableVector();
        if(notBefore!=null){
            DERTaggedObject notBeforeTagged = new DERTaggedObject(0, notBefore);
            privateKeyUsagePeriod.add(notBeforeTagged);
        }        
        if(notAfter!=null){
            DERTaggedObject notAfterTagged = new DERTaggedObject(1, notAfter);
            privateKeyUsagePeriod.add(notAfterTagged);
        }
        return new DERSequence(privateKeyUsagePeriod);
    }
    /**Returns a description of the value.*/
    public String toString(){
        String notBeforeString = "NOT PRESENT";
        String notAfterString = "NOT PRESENT";
        try{
            if(notBefore!=null){
                notBeforeString = notBefore.getDate().toGMTString();
            }
            if(notAfter!=null){
                notAfterString = notAfter.getDate().toGMTString();
            }
        }
        catch(java.text.ParseException e){
            e.printStackTrace();
            notBeforeString = "PARSING ERROR";
            notAfterString = "PARSING ERROR";
        }
        return String.format("PrivateKeyUsagePeriod: [not Before: %s], [not After: %s]",
                notBeforeString, notAfterString);
    }
   
}
