/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org) 
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509.extensions;

import be.cardon.asn1.x509.*;
import org.bouncycastle.asn1.x509.qualified.BiometricData;

import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.ASN1TaggedObject;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERSequence;

import org.bouncycastle.asn1.x509.qualified.QCStatement;

import java.util.Vector;
import java.util.Iterator;

/**Extension {@code QCStatements} (RFC 3039).

   <p>Documentation from RFC <b>3039</b>:</p>
   <p>This section defines an extension for inclusion of defined statements
   related to Qualified Certificates.</p>

   <p>ypical statement suitable for inclusion in this extension MAY be a
   statement by the issuer that the certificate is issued as a Qualified
   Certificate in accordance with a particular legal system (as
   discussed in Section 2.2).</p>

   <p>Other statements suitable for inclusion in this extension MAY be
   statements related to the applicable legal jurisdiction within which
   the certificate is issued.  As an example this MAY include a maximum
   reliance limit for the certificate indicating restrictions on CA's
   liability.</p>

   <p>Each statement SHALL include an object identifier for the statement
   and MAY also include optional qualifying data contained in the
   statementInfo parameter.</p>

   <p>If the statementInfo parameter is included then the object identifier
   of the statement SHALL define the syntax and SHOULD define the
   semantics of this parameter.  If the object identifier does not
   define the semantics, a relying party may have to consult a relevant
   certificate policy or CPS to determine the exact semantics.</p>

   <p>This extension may be critical or non-critical.  If the extension is
   critical, this means that all statements included in the extension
   are regarded as critical.</p>
   <pre>
      qcStatements  EXTENSION ::= {
          SYNTAX             QCStatements
          IDENTIFIED BY      id-pe-qcStatements }

      id-pe-qcStatements     OBJECT IDENTIFIER ::= { id-pe 3 }

      QCStatements ::= SEQUENCE OF QCStatement

      QCStatement ::= SEQUENCE {
          statementId   QC-STATEMENT.&Id({SupportedStatements}),
          statementInfo QC-STATEMENT.&Type
          ({SupportedStatements}{@statementId}) OPTIONAL }

      SupportedStatements QC-STATEMENT ::= { qcStatement-1,...}
      </pre>

    <p><b>Predefined Statements</b></p>

   <p>This profile includes one predefined object identifier (id-qcs-
   pkixQCSyntax-v1), identifying conformance with syntax and semantics
   defined in this profile.  This Qualified Certificate profile is
   referred to as version 1.</p>
      <pre>
      qcStatement-1 QC-STATEMENT ::= { SYNTAX SemanticsInformation
          IDENTIFIED BY id-qcs-pkixQCSyntax-v1 }
      --  This statement identifies conformance with syntax and
      --  semantics defined in this Qualified Certificate profile
      --  (Version 1). The SemanticsInformation may optionally contain
      --  additional semantics information as specified.

      SemanticsInformation ::= SEQUENCE {
          semanticsIdentifier        OBJECT IDENTIFIER   OPTIONAL,
          nameRegistrationAuthorities NameRegistrationAuthorities
                                                          OPTIONAL }
          (WITH COMPONENTS {..., semanticsIdentifier PRESENT}|
           WITH COMPONENTS {..., nameRegistrationAuthorities PRESENT})

      NameRegistrationAuthorities ::=  SEQUENCE SIZE (1..MAX) OF
          GeneralName
     </pre>

   <p>The SementicsInformation component identified by id-qcs-
   pkixQCSyntax-v1 MAY contain a semantics identifier and MAY identify
   one or more name registration authorities.</p>

   <p>The semanticsIdentifier component, if present, SHALL contain an OID,
   defining semantics for attributes and names in basic certificate
   fields and certificate extensions.  The OID may define semantics for
   all, or for a subgroup of all present attributes and/or names.</p>

   <p>The NameRegistrationAuthorities component, if present, SHALL contain
   a name of one or more name registration authorities, responsible for
   registration of attributes or names associated with the subject.  The
   association between an identified name registration authority and
   present attributes MAY be defined by a semantics identifier OID, by a
   certificate policy (or CPS) or some other implicit factors.</p>

   <p>If a value of type SemanticsInformation is present in a QCStatement
   then at least one of the fields semanticsIdentifier and
   nameRegistrationAuthorities must be present, as indicated.</p>
 */
public class QCStatements extends ASN1Encodable{
    
    Vector<QCStatement> qCStatements;
    
    /** Creates a new instance of BiometricSyntax */
    public QCStatements(ASN1Sequence seq) {
        qCStatements = new Vector<QCStatement>();
        for(int i =0;i<seq.size();i++){
            qCStatements.add(QCStatement.getInstance(seq.getObjectAt(i)));
        }
    }
    
    public static QCStatements getInstance(Object obj){
        if(obj instanceof QCStatements){
            return (QCStatements)obj;
        }
        if(obj instanceof ASN1Sequence){
            return new QCStatements((ASN1Sequence)obj);
        }
        throw new IllegalArgumentException("invalid sequence");
    }

    public Vector<QCStatement> getQCStatement() {
        return qCStatements;
    }

    public DERObject toASN1Object(){
        ASN1EncodableVector vec = new ASN1EncodableVector();
        Iterator<QCStatement> it = qCStatements.iterator();
        while(it.hasNext()){
            vec.add(it.next().toASN1Object());
        }
        return new DERSequence(vec);
    }
    
}
