/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org) 
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509.extensions;

import be.cardon.asn1.x509.GeneralName;
import be.cardon.asn1.x509.AccessDescription;

import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1EncodableVector;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.DERSequence;

import java.util.Iterator;
import java.util.Vector;
/**Extension {@code SubjectInformationAccess}.

   <p>Documentation from <b>RFC 3280</b>:</p>
   <p>The subject information access extension indicates how to access
   information and services for the subject of the certificate in which
   the extension appears.  When the subject is a CA, information and
   services may include certificate validation services and CA policy
   data.  When the subject is an end entity, the information describes
   the type of services offered and how to access them.  In this case,
   the contents of this extension are defined in the protocol
   specifications for the suported services.  This extension may be
   included in subject or CA certificates, and it MUST be non-critical.</p>
   <pre>
   id-pe-subjectInfoAccess OBJECT IDENTIFIER ::= { id-pe 11 }

   SubjectInfoAccessSyntax  ::=
           SEQUENCE SIZE (1..MAX) OF AccessDescription

   AccessDescription  ::=  SEQUENCE {
           accessMethod          OBJECT IDENTIFIER,
           accessLocation        GeneralName  }
   </pre>
   <p>Each entry in the sequence {@code SubjectInfoAccessSyntax} describes the
   format and location of additional information provided by the subject
   of the certificate in which this extension appears.  The type and
   format of the information is specified by the {@code accessMethod} field; the
   {@code accessLocation} field specifies the location of the information.  The
   retrieval mechanism may be implied by the {@code accessMethod} or specified
   by {@code accessLocation}.</p>

   <p>This profile defines one access method to be used when the subject is
   a CA, and one access method to be used when the subject is an end
   entity.  Additional access methods may be defined in the future in
   the protocol specifications for other services.</p>

   <p>The {@code id-ad-caRepository} OID is used when the subject is a CA, and
   publishes its certificates and CRLs (if issued) in a repository.  The
   {@code accessLocation} field is defined as a {@code GeneralName}, which can take
   several forms.  
   <ul>
   <li>Where the information is available via http, ftp, or
   ldap, {@code accessLocation} MUST be a {@code uniformResourceIdentifier}.</li>
   <li>Where the information is available via the directory access protocol (dap),
   {@code accessLocation} MUST be a {@code directoryName}.</li>
   <li>When the information is available via electronic mail, 
   {@code accessLocation} MUST be an {@code rfc822Name}.</li>
   <li>The semantics of other name forms of of {@code accessLocation} (when
 * {@code accessMethod} is {@code id-ad-caRepository}) are not defined by this
   specification.</li>
   </ul>
   </p>

   <p>The {@code id-ad-timeStamping} OID is used when the subject offers
   timestamping services using the Time Stamp Protocol defined in
   [PKIXTSA].  
   <ul>
   <li>Where the timestamping services are available via http or
   ftp, {@code accessLocation} MUST be a {@code uniformResourceIdentifier}.</li>
   <li>Where the timestamping services are available via electronic mail,
   {@code accessLocation} MUST be an {@code rfc822Name}.  </li>
   <li>Where timestamping services are available using TCP/IP, 
   the {@code dNSName} or {@code ipAddress} name forms may
   be used.  </li>
   <li>The semantics of other name forms of {@code accessLocation} (when
   {@code accessMethod} is {@code id-ad-timeStamping}) are not defined by this
   specification.</li>
   </ul>
   </p>

   <p>Additional access descriptors may be defined in other PKIX
   specifications.</p>
   <pre>
   id-ad OBJECT IDENTIFIER ::= { id-pkix 48 }

   id-ad-caRepository OBJECT IDENTIFIER ::= { id-ad 5 }

   id-ad-timeStamping OBJECT IDENTIFIER ::= { id-ad 3 }
   </pre>
 * @see be.cardon.asn1.x509.AccessDescription
 * @author crackapat
 */
public class SubjectInformationAccess 
    extends ASN1Encodable
{
    private Vector<AccessDescription> accessDescriptionVector;

    /**Returns an instance from a {@code ASN1Sequence}, or a
     *{@code AuthorityInformationAccess} object.*/
    public static SubjectInformationAccess getInstance(
        Object  obj)
    {
        if (obj instanceof SubjectInformationAccess)
        {
            return (SubjectInformationAccess)obj;
        }
        else if (obj instanceof ASN1Sequence)
        {
            return new SubjectInformationAccess((ASN1Sequence)obj);
        }

        throw new IllegalArgumentException("unknown object in factory");
    }
 
    /**Creates a new instance from a {@code ASN1Sequence} object.*/
    public SubjectInformationAccess(
        ASN1Sequence   seq)
    {
        accessDescriptionVector = new Vector<AccessDescription>();
        
        for (int i = 0; i != seq.size(); i++)
        {
            accessDescriptionVector.add(AccessDescription.getInstance(seq.getObjectAt(i)));
        }
    }

    /**
     * Creates a new instance with the given {@code accessDescription} vector.
     */
    public SubjectInformationAccess(
        Vector<AccessDescription> accessDescriptionVector)
    {
        this.accessDescriptionVector = accessDescriptionVector;
    }

    /**Returns a vector with the {@code AccessDescription} objects.*/
    public Vector<AccessDescription> getAccessDescriptionVector() {
        return accessDescriptionVector;
    }

    /**@inheritDoc
     */
    public DERObject toASN1Object()
    {
        ASN1EncodableVector vec = new ASN1EncodableVector();
        Iterator<AccessDescription> it = accessDescriptionVector.iterator();
        while(it.hasNext()){
             vec.add(it.next().toASN1Object());
        }
        
        return new DERSequence(vec);
    }
}
