/*
Copyright (c) 2000-2006 The Legion Of The Bouncy Castle (http://www.bouncycastle.org) 
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.asn1.x509.extensions;

import be.cardon.asn1.x509.SubjectPublicKeyInfo;
import org.bouncycastle.asn1.ASN1Encodable;
import org.bouncycastle.asn1.ASN1OctetString;
import org.bouncycastle.asn1.ASN1TaggedObject;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DEROctetString;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * Extension {@code SubjectKeyIdentifier}.
   <p>Documentation from <b>RFC 3280</b>:<p>
   <p>The subject key identifier extension provides a means of identifying
   certificates that contain a particular public key.</p>

   <p>To facilitate certification path construction, this extension MUST
   appear in all conforming CA certificates, that is, all certificates
   including the basic constraints extension (section 4.2.1.10) where
   the value of cA is TRUE.  The value of the subject key identifier
   MUST be the value placed in the key identifier field of the Authority
   Key Identifier extension (section 4.2.1.1) of certificates issued by
   the subject of this certificate.<p>

   <p>For CA certificates, subject key identifiers SHOULD be derived from
   the public key or a method that generates unique values.  Two common
   methods for generating key identifiers from the public key are:</p>
   <ul>
      <li>(1) The {@code keyIdentifier} is composed of the 160-bit SHA-1 hash of the
      value of the {@code BIT STRING subjectPublicKey} (excluding the tag,
      length, and number of unused bits).</li>

      <li>(2) The {@code keyIdentifier} is composed of a four bit type field with
      the value 0100 followed by the least significant 60 bits of the
      SHA-1 hash of the value of the {@code BIT STRING subjectPublicKey}
      (excluding the tag, length, and number of unused bit string bits).</li>
   </ul>
   <p>One common method for generating unique values is a monotonically
   increasing sequence of integers.</p>

   <p>For end entity certificates, the subject key identifier extension
   provides a means for identifying certificates containing the
   particular public key used in an application.  Where an end entity
   has obtained multiple certificates, especially from multiple CAs, the
   subject key identifier provides a means to quickly identify the set
   of certificates containing a particular public key.  To assist
   applications in identifying the appropriate end entity certificate,
   this extension SHOULD be included in all end entity certificates.</p>

   <p>For end entity certificates, subject key identifiers SHOULD be
   derived from the public key.  Two common methods for generating key
   identifiers from the public key are identified above.</p>

   <p>Where a key identifier has not been previously established, this
   specification RECOMMENDS use of one of these methods for generating
   keyIdentifiers.  Where a key identifier has been previously
   established, the CA SHOULD use the previously established identifier.</p>

   <p>This extension MUST NOT be marked critical.</p>

   <p>The ASN.1 definitions are:</p>
   <pre>
   id-ce-subjectKeyIdentifier OBJECT IDENTIFIER ::=  { id-ce 14 }

   SubjectKeyIdentifier ::= KeyIdentifier
  
   KeyIdentifier ::= OCTET STRING
   </pre>
 *@see AuthorityKeyIdentifier
 */
public class SubjectKeyIdentifier
    extends ASN1Encodable
{
    private byte[] keyidentifier;
    
    /** Creates an new object from an existing {@literal ASN.1} tagged object.*/
    public static SubjectKeyIdentifier getInstance(
        ASN1TaggedObject obj,
        boolean          explicit)
    {
        return getInstance(ASN1OctetString.getInstance(obj, explicit));
    }

     /** Creates an new object from an existing {@code ASN1OctetString} 
     * or {@code SubjectKeyIdentifier} object.
     *@throws IllegalArgumentException if the object is invalid, or {@code null}.
     */
    public static SubjectKeyIdentifier getInstance(
        Object obj)
    {
        if(obj == null){
             throw new IllegalArgumentException("Null object");
        }
        
        if (obj instanceof SubjectKeyIdentifier) 
        {
            return (SubjectKeyIdentifier)obj;
        }
        
        if (obj instanceof ASN1OctetString) 
        {
            return new SubjectKeyIdentifier((ASN1OctetString)obj);
        }
        
        throw new IllegalArgumentException("Invalid SubjectKeyIdentifier: " + obj.getClass().getName());
    }
    
    /** Creates an new object with the given {@code KeyIdentifier}.*/
    public SubjectKeyIdentifier(
        ASN1OctetString  keyid)
    {
        this.keyidentifier=keyid.getOctets();

    }    
    /** Creates an new object with the given {@code KeyIdentifier}.*/
    public SubjectKeyIdentifier(
        byte[] keyid)
    {
        this.keyidentifier=keyid;
    }
    


    /**
     * Calulates the {@code KeyIdentifier} using a SHA1 hash over the BIT STRING
     * from SubjectPublicKeyInfo as defined in RFC 3280.
     *@throws NoSuchAlgorithmException if no SHA-1 {@code MessageDigest} service found.
     **/
    public SubjectKeyIdentifier(
        SubjectPublicKeyInfo    spki)
        throws NoSuchAlgorithmException
    {
        MessageDigest digest = MessageDigest.getInstance("SHA-1");
        byte[] bytes = spki.getPublicKeyData().getBytes();
        byte[] resBuf = digest.digest(bytes);
        this.keyidentifier = resBuf;
    }

    /**Returns the {@code KeyIdentifier}.*/
    public byte[] getKeyIdentifier()
    {
        return this.keyidentifier;
    }

    /**@inheritDoc
     */
    public DERObject toASN1Object()
    {
        return new DEROctetString(keyidentifier);
    }
}

